/*
	Copyright (c) 2005, Quentin Lequy and Romain Gaucher
	All rights reserved.

	Redistribution and use in source and binary forms, with or 
	without modification, are permitted provided that the following 
	conditions are met:

		* Redistributions of source code must retain the above copyright 
		  notice, this list of conditions and the following disclaimer.

		* Redistributions in binary form must reproduce the above copyright 
		  notice, this list of conditions and the following disclaimer in the 
		  documentation and/or other materials provided with the distribution.

		* Neither the name of Quentin Lequy or Romain Gaucher nor the names 
		  of its contributors may be used to endorse or promote products 
		  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
	AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef __EXPR_H_
#define __EXPR_H_

#include <list>
#include <iostream>

#include "ExprNode.h"

namespace Modelib {

class Num;
class NumArray;
class NumVar;
class NumVarArray;
class LagrangianParam;


///Classe representant les expressions linaires.
/**Elle va etre utilise pour dcire la fonction objectif, les containtes et les bornes. \n
   - Elle est constitu d'une liste de classe "ExprNode". \n
   - L'operateur operator+=(const ExprNode & _node) permet la construction iterative de ces expressions. \n
   - Une expressions peut tre construite  partir de Num et de NumVar et peut tre dfinir par l'ajout
   d'une expressions  elle avec diffrents oprateurs \n
   - Elle est dclar friend avec la classe Model car elle n'a volontairement pas de geters, de seters ou d'iterateurs
   sur la liste d'ExprNode qui est prive. La classe Model est ainsi la seule qui peut modifi sa strcuture
   interne. Pour les mme raisons l'oprateur de flux <<  besoin d'tre dclarer friend. \n
   - Les expressions tant simplifies a chaque fois, il serait peut-tre bon de rajouter une fonction qui permet de
   construire une fonction sans la simplifier
 */

class Expr
{
  std::list<ExprNode> nodeList;
  protected:
    /**Oprateur d'ajout de node
       @param _node un noeud de l'expression du type ExprNode
       @return une refrence pour pouvoir enchainer les opraeurs +=
     */
    Expr & operator+=(const ExprNode & _node)
    {
      nodeList.push_back(_node);
      isSimplified = false;
      return *this;
    }
    
    void Simplify(bool AvecLagrangianParam = false,bool SansConstantes = false);
    
    bool isSimplified;
    bool isParametric;
    bool sansConstantes;

    friend class Model;
  public:
    Expr() {}
    Expr( const NumVar & numVar);
    Expr( float );
    Expr( double );
    Expr( int );
    Expr( unsigned );
    Expr( const Num & num);
    Expr( const Num & num, const NumVar & numVar);
    Expr( const Num & num, const NumVar & numVar, const LagrangianParam & mult);
    /**Constructeur de copie*/
    Expr( const Expr & _expr) { *this = _expr; }
    
    /**Oprateur de recopie*/
    Expr & operator= ( const Expr & _expr)
    {
        if( this != &_expr )
        {
            nodeList = _expr.nodeList;
            isSimplified =  _expr.isSimplified;
            isParametric = _expr.isParametric;
            sansConstantes = _expr.sansConstantes;
        }
        return *this;
    }
    
    Expr & operator+=( const Expr & );
    Expr & operator-=( const Expr & );

    Expr & operator*=(const Num &);

    Expr & ASS( const Num & num, const NumVar & numVar );
    Expr & ASS( const Num & num, const NumVar & numVar, const LagrangianParam & mult );

    /**Permet de connaitre la taille d'une expression
      @return Le nombre de bloc lementaire composant l'expression
    */
    unsigned Size() { return nodeList.size(); }
    bool IsNum();
    bool IsVar();
    float GetConstant(bool avecLagrangianParam = true) const;
    float Evalue(bool avecLagrangianParam = true) const;

    friend std::ostream & operator<< ( std::ostream & flux, const Expr & expr);
    friend std::ostream& operator<<(std::ostream& o,const Model& m);

    ~Expr() {}
}; 

//Operateurs de constructions d'expressions
Expr operator*( const Num & , const NumVar &);
Expr operator*( const NumVar & , const Num &);
Expr operator*( const NumVarArray & , const NumArray &);
Expr operator*( const NumArray & , const NumVarArray &);
Expr operator*( const Expr &, const Num & );
Expr operator*( const Num & , const Expr &);
Expr operator+( const Expr &, const Expr &);
Expr operator-( const Expr &, const Expr &);


std::ostream & operator<< (std::ostream & flux,const Expr & expr);

Expr Sum(const NumVarArray & );

}

#endif
