/*
	Copyright (c) 2005, Quentin Lequy and Romain Gaucher
	All rights reserved.

	Redistribution and use in source and binary forms, with or 
	without modification, are permitted provided that the following 
	conditions are met:

		* Redistributions of source code must retain the above copyright 
		  notice, this list of conditions and the following disclaimer.

		* Redistributions in binary form must reproduce the above copyright 
		  notice, this list of conditions and the following disclaimer in the 
		  documentation and/or other materials provided with the distribution.

		* Neither the name of Quentin Lequy or Romain Gaucher nor the names 
		  of its contributors may be used to endorse or promote products 
		  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
	AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef __MODELE_H
#define __MODELE_H

#include "Config.h"
#include "Fraction.h"
#include "Constant.h"
#include "MuteVar.h"
#include "Paire.h"
#include "MatriceCreuse.h"
#include "Expr.h"
#include "ConstraintBuilder.h"
#include "InternalConstraint.h"
#include "Constraint.h"
#include "SolverAPI.h"
#include "LagrangianParam.h"
#include "Nom.h"

#ifdef USE_LATEX_FORMAT
       #include "latexFile.h"
#endif

#ifdef USE_HTML_FORMAT
       #include "htmlFile.h"
#endif

#include <map>
#include <string>
#include <vector>
#include <iostream>

namespace Modelib {

///Classe rpresentant le programme lineaire en lui meme
/**Elle contient:
  -un entier permettant l'attribution d'un numero a chaque variable
  -un entier permettant l'attribution d'un numero a chaque contrainte
  -une base de donnee sur les variables du vecteur inconnu
  -une Matrice creuse representant les contraintes lineaires
  -une base de donnee sur les contraintes lineaires
  -une base de donnee de commentaires
  -un booleen caracterisant le type de fonction objectif
  -un booleen specifiant si le programme linaire est relache
  -un booleen specifiant si l'utilisateur desire un affichage fractionnaire
  -un booleen specifiant si la solution est optimale
  -un booleen specifiant si le probleme a ete resolu
  -un booleen specifiant si le probleme a ete normalise
  -la fonction objectif sous la forme d'une expression
  -la valeur de la fonction objectif apres resolution
  -le nom de la fonction objectif
  -le nom du programme linaire
  -une interface avec les solveurs
  -un convertisseur pour l'aaffichage fractionnaire

   Ce modele est independant d'un environnement
   Ce modele peut etre modifi par la cration de variable et l'ajout de contraintes. \n
   Les variables peuvent etre modifiee par le biais de la classe NumVar. \n
   L'operateur << permet d'afficher le probleme sous la forme d'un fichier LP. \n
  */

class Model
{
    VarId hacheVarId;
    VarId hacheConstraintId;
    std::map< VarId, Paire > unknowMap;
    MatriceCreuse<float> A; // AX <= B, A*X = B, A*X < B
    map<VarId, InternalConstraint> constraintMap;
    map<VarId, std::string> comments;

    bool isMax;
    bool isRelaxed;
    bool isFract;
    bool isOptimal;
    bool isSolved;
    bool isNormalized;
    unsigned MIPVar;

    Expr objectif;
    float objectifActivity;
    std::string objectifName;
    std::string problemName;

    // gestion des solvers
    SolverAPI solverAPI;
    // Fraction...
    FractionConvert _fract;

// Pointeurs de fonctions pour le polymorphisme
// pour la definission de Python

#ifdef COMPILE_FOR_PYTHON
  public:
	Constraint GetConstraint_1(VarId id) {return GetConstraint(id);}
	Constraint GetConstraint_2(const std::string &_str) {return GetConstraint(_str);}
#endif

  protected:
    friend class LagrangianParam;
    float GetLagrangianParamValue( VarId ) const;
    void SetLagrangianParamValue(VarId , float);

    friend class Constraint;


    friend class NumVar;

    VarId AddVar( const MuteVar & muteVar, const std::string &);
    VarId GetVarId(const std::string &) const;
    MuteVar GetMuteVar( VarId varId) const;
    NumVar GetNumVar( VarId varId ) const;
    std::string GetName( VarId varId ) const;
    void SetName( VarId varId, const std::string & name);
    float GetMuteVarLB( VarId varId) const;
    void SetMuteVarLB( VarId varId, float _value);
    float GetMuteVarUB( VarId varId) const;
    void SetMuteVarUB( VarId varId, float _value);
    void SetMuteVarLBStrict( VarId varId, bool _value);
    void SetMuteVarUBStrict( VarId varId, bool _value);
    MuteVar::VarType GetType( VarId varId ) const;
    void SetType( VarId varId, MuteVar::VarType _type);

    VarId AddExpr( Expr & expr, MatriceCreuse<float> & A);
    Expr GetRelaxedConstraintExpr( VarId id, bool relaxed = true );

    void ComputeObjectif();

    std::ostream & SurchargeFlux(std::ostream&) const;
    std::ostream & SurchargeFluxRelaxed(std::ostream&) const;
    
    std::ostream & SurchargeMPSFlux(std::ostream&,bool verbose) const;

    VarId GetConstraintId(const std::string &);

  public:
    /**Constructeur par dfaut*/
    Model()
    : hacheVarId(1),hacheConstraintId(1),
      isMax(true),isRelaxed(false),isFract(false),
      isOptimal(false),isSolved(false),isNormalized(false),
      MIPVar(0),objectifActivity(0)
      {}
    Model(const Model &);
    Model & operator=(const Model &);

    Constraint Add( const ConstraintBuilder &, const std::string & nom = "");
    Constraint Add( const ConstraintBuilder & constraint, const Nom & _nom)
    {
    	return Add(constraint,_nom());
    }

    void Maximize(const Expr & expr, const std::string & nom = "");
    void Minimize(const Expr & expr, const std::string & nom = "");
    Expr GetObjectif() const { return objectif;}

    void SetProblemName( const std::string name = "") { problemName = name;}
    string GetProblemName( ) const { return problemName;}

    inline bool IsRelaxed() const {return isRelaxed;}
    inline void SetRelaxed(bool _isRelaxed) { isRelaxed = _isRelaxed; }

    bool RelaxInt2Float ( NumVar numVar = NumVar() );
    bool RelaxBool2Float( NumVar numVar = NumVar() );

    void SetNumVarValues(const std::list<std::string> & noms, const std::list<float> & values);
    void SetConstraintValues(const std::list<std::string> & noms, const std::list<float> & values);
    void GetInfluentLagrangianParam( std::list<LagrangianParam> & listViolated, std::list<LagrangianParam> & listSlacked);
    std::vector<LagrangianParam> GetAllLagrangianParams();
    float EvalConstraint(VarId id ) const;

    void UnrelaxModel();

    //void Standardize(bool withoutStrictConstraintConversion = true,bool withOnlyPositiveVariable = true);

    std::ostream & Out( std::ostream & flux = std::cout, const std::string & newline = "" );
    std::ostream & OutDimensions( std::ostream & flux = std::cout, const std::string & newline = "" );
    std::ostream & OutNumVarValues( bool withoutNullValue = true,std::ostream & flux = std::cout, const std::string & newline = "" ) const;
    std::ostream & OutConstraintValues(std::ostream & flux = std::cout, const std::string & newline = "" ) const;
    friend std::ostream& operator<<(std::ostream&, const Model&);
    friend std::ostream & ToMPS(std::ostream&, const Model&,bool verbose=true);

    Constraint GetConstraint(VarId id) const;
    Constraint GetConstraint(const std::string &);
   	bool RelaxConstraint( VarId id, float lagrangianValue = -Infinity );
    bool Relax(const std::string & name, float lagrangianValue = -Infinity);
    void RenameConstraint( VarId id, const std::string & name);

    VarId GetMaxVarId() const { return hacheVarId;}
    VarId GetMaxConstraintId() const { return hacheConstraintId;}

    // Chargement d'un fichier LP
    bool LoadLP(const std::string& , bool verbose = false);
    bool LoadMPS(const std::string& , bool verbose = false);

    void RemoveVariable(VarId varId);
    void RemoveConstraint(VarId varId);
    void AddToConstraint(VarId,const Expr &);
    void SetSecondMember(VarId,float);
    float GetSecondMember(VarId) const;
    void Clear();
    void Normalize(bool withoutStrict = true);
    void PrintDual(const std::string&) const;
    Model GetDual() const;

    // interface du solver
    void  InitSolver(API _API=GLPK,const std::string& _fLP="./tmp.solver.lp", const std::string& _path="")
    {
    	solverAPI.Init(_fLP,_path);
    	SetAPI(_API);
    }

    float Solve();
		 // Retourner les informations (Minimisation/Maximisation) et la phrase que sort GLPSOL
		 // genre: 'INTEGER OPTIMAL'
    void  SolveInfos(std::string& _operation, std::string& _informations)
    {
      solverAPI.Infos(_operation,_informations);
    }

    // Retourne l'occupation de la matrice
    double  MatriceOccupation()
    {
      return (double(A.GetSize()) / double(A.GetNbCols() * A.GetNbLignes()));
    }

	inline float getObjValue() {
		return objectifActivity;
	}

    void  SetAPI(API _API);
    void  PushOptions(const std::string& _str) { solverAPI.Options(_str); }
    void  RemoveOptions() {solverAPI.RemoveOptions();}

    void  ShowMe(  bool withoutNullValue = true );

    void  PrintFraction() {isFract = true;}
    void  EndFraction()   {isFract = false;}
    
    std::string AddComment( const std::string & comment );
    void SetComment( VarId id, const std::string &  comment);
    std::string GetComment( VarId id ) const;

#ifdef USE_HTML_FORMAT
    void SortieHTML( const std::string & fileName, const std::string& matrixPNG="");
#endif

#ifdef USE_LATEX_FORMAT
    void SortieLATEX( const std::string & fileName);
#endif

#ifdef EXPORT_SPARSE_MATRIX_PNG
    void ExportSparseMatrixPNG(const std::string& _nom="./MatriceCreuse.png") const;
#endif


};

#ifdef COMPILE_FOR_PYTHON
	void export_Model();
#endif

}

#endif
