/*
	Copyright (c) 2005, Quentin Lequy and Romain Gaucher
	All rights reserved.

	Redistribution and use in source and binary forms, with or 
	without modification, are permitted provided that the following 
	conditions are met:

		* Redistributions of source code must retain the above copyright 
		  notice, this list of conditions and the following disclaimer.

		* Redistributions in binary form must reproduce the above copyright 
		  notice, this list of conditions and the following disclaimer in the 
		  documentation and/or other materials provided with the distribution.

		* Neither the name of Quentin Lequy or Romain Gaucher nor the names 
		  of its contributors may be used to endorse or promote products 
		  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
	AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef __MUTEVAR_H
#define __MUTEVAR_H

namespace Modelib {

//!Classe de base reprsentant les variables dans le modele
/**
  Elle est compose en priv:
  - de deux boolens "isBool" et "isInt" permettant de definir son type parmi VarType(economie de place)
  - de deux boolens "isLBStrict" et "isUBStrict" permettant de definir si les bornes du domaine de dfinition de
    la variable sont ouvertes ou fermes
  - d'un rel "value" stockant la value de la variable aprs rsolution
  - de deux rels "lowerBound" et "upperBound" dlimitant le domaine de dfintion de la variable
 */

class MuteVar
{
    bool isBool; //!< La variable est-elle boolenne?
    bool isInt;  //!< La variable est-elle entire?
    bool isLBStrict; //!< La borne infrieure est-elle stricte?
    bool isUBStrict; //!< La borne suprieure est-elle stricte?
    bool isRelaxed;  //!< La variable est elle relaxe en float?
    float value;    //!< Value de la variable aprs rsolution
    float lowerBound; //!<Value de la borne infrieure
    float upperBound; //!<Value de la borne infrieure

  public:
    typedef enum VarType{FLOAT=0,INT,BOOL};

    /** Contructeur de base et par dfaut */
    MuteVar( MuteVar::VarType type = MuteVar::FLOAT,
             float _value = 0.0f,
             float _lB = 0.0f,
             float _uB = 0.0f )
    :isBool(type==MuteVar::BOOL),isInt(type==MuteVar::INT),
     isLBStrict(false),isUBStrict(false),isRelaxed(false),
     value(_value),lowerBound(_lB),upperBound(_uB)
    {}

    /** Contructeur de recopie */
    MuteVar(const MuteVar & _mv)
    {
      *this = _mv;
    }

    /**Oprateur de recopie */
    MuteVar & operator= (const MuteVar & _mv)
    {
      isBool = _mv.isBool;
      isInt = _mv.isInt;
      isLBStrict = _mv.isLBStrict;
      isUBStrict = _mv.isUBStrict;
      isRelaxed = _mv.isRelaxed;
      value = _mv.value;
      lowerBound = _mv.lowerBound;
      upperBound = _mv.upperBound;
      return *this;
    };

    /**Oprateur d'assignement*/
    MuteVar & operator= (float _value)
    {
      value = _value;
      return *this;
    };

    /**Geter de la value de la variable
       @return La value de la variable aprs rsolution
      */
    float GetValue() const {return value;}
    /**Geter de la borne infrieure
       @return lowerBound
      */
    float GetLowerBound() const {return lowerBound;}
    /**Geter de la borne suprieure
       @return upperBound
      */
    float GetUpperBound() const {return upperBound;}
    /**La borne infrieure est-elle stricte?
       @return isLBStrict
      */
    bool IsLowerBoundStrict() const {return isLBStrict;}
    /**La borne suprieure est-elle stricte?
       @return isUBStrict
      */
    bool IsUpperBoundStrict() const {return isUBStrict;}
    
    /**Geter du type de la variable
       @return Le type de la variable
      */
    VarType GetType() const
    {
      if(isBool) return MuteVar::BOOL;
      if(isInt) return MuteVar::INT;
      return MuteVar::FLOAT;
    }

    /**Seter du type de la variable
       @param Mute::Vartype le type de la variable
      */
    void SetType( MuteVar::VarType type)
    {
      isBool = false;
      isInt = false;
      isBool = (type==MuteVar::BOOL);
      isInt = (type==MuteVar::INT);
    }

    /**Seter de la borne infrieure
       @param _value la nouvelle value de la borne infrieure lowerBound
      */
    void SetLowerBound( float _value) { lowerBound = _value;}
    /**Geter de la borne infrieure
      @param _value la nouvelle value de la borne uprieure upperBound
      */
    void SetUpperBound( float _value) { upperBound = _value;}
    /**Seter pour dfinir la borne infrieure strict
       @param _value la nouvelle value isLBStrict
      */
    void SetIsLBStrict( bool _value = true ) { isLBStrict = _value; }
    /**Seter pour dfinir la borne suprieure strict
       @param _value la nouvelle value isUBStrict
      */
    void SetIsUBStrict( bool _value = true ) { isUBStrict = _value; }

    bool IsRelaxed() const { return isRelaxed;}
    void SetRelaxed( bool _isRelaxed = true ) { isRelaxed = _isRelaxed;}

    ~MuteVar()
    {
    }
  
};

}

#endif
