/*
	Copyright (c) 2005, Quentin Lequy and Romain Gaucher
	All rights reserved.

	Redistribution and use in source and binary forms, with or 
	without modification, are permitted provided that the following 
	conditions are met:

		* Redistributions of source code must retain the above copyright 
		  notice, this list of conditions and the following disclaimer.

		* Redistributions in binary form must reproduce the above copyright 
		  notice, this list of conditions and the following disclaimer in the 
		  documentation and/or other materials provided with the distribution.

		* Neither the name of Quentin Lequy or Romain Gaucher nor the names 
		  of its contributors may be used to endorse or promote products 
		  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
	AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef __NUMVARARRAY_H
#define __NUMVARARRAY_H

#include "Config.h"
#include "Constant.h"
#include "MuteVar.h"

#include <vector>
#include <string>
#include <stdexcept>

namespace Modelib {

//Pour viter les dfinitions circulaire
class Model;
class NumVar;

//!Classe reprsentant les vecteurs d'inconnues
/**
  Son utilit rside dans la dclarations de familles de variables.
  Elles ont le mme nom  un indice prs.
*/

class NumVarArray
{
  protected:
  Model * modele;                   //!< Pointeur sur le modele de dfinition
  std::vector<NumVar *> varArray;    //!< Vecteur de pointeurs sur des variables
  MuteVar::VarType type;             //!< Type general du vecteur

  public:
    /**Constructeur par dfaut */
    NumVarArray():modele(0),type(MuteVar::FLOAT) {}

    NumVarArray( Model & _modele,
                 unsigned size = 0,
                 float _lb= 0,
                 float _ub= Infinity,
                 MuteVar::VarType _type = MuteVar::FLOAT,
                 const std::string & nom = "unknow",
                 const std::string & fin = "");

    NumVarArray( const NumVarArray & nva );
    
    NumVarArray( const std::vector<NumVar> & nva );

    NumVarArray & operator= ( const NumVarArray & nva );

    /**Geter de taille
       @return La taille du vecteur de variables
    */
    unsigned Size() const { return varArray.size();}
    /**Oprateur constant d'indice
       @param n l'indice du tableau pour lequel on souhaite rcuprer la variable.
       @return Une rfrence sur la variable.
    */
    NumVar& operator[] (unsigned n) const
    {
        if( n < varArray.size() )
            return *(varArray[n]);
        else
            throw std::out_of_range("NumVarray::operator[] : index out of range");
    }
    /**Geteur de la variable d'indice n
       @param n l'indice du tableau pour lequel on souhaite rcuprer la variable.
       @return Un pointeur sur la variable.
    */

#ifndef COMPILE_FOR_PYTHON    
	NumVar * Get(unsigned n) const 
    {
        if( n < varArray.size() )
            return varArray[n];
        else
            throw std::out_of_range("NumVarray::Get : index out of range");
    }
#else
	NumVar&   Get(unsigned n) const 
    {
        if( n < varArray.size() )
            return *(varArray[n]);
        else
            throw std::out_of_range("NumVarray::Get : index out of range");
    }
#endif
    /**Geter de type
       @return Le type du vecteur de variables.
    */
    MuteVar::VarType GetType() const { return type;}

    ~NumVarArray();

#ifdef COMPILE_FOR_PYTHON
    NumVar&   GetItem(unsigned n) const      {return *(varArray[n]);}
    void     SetItem(NumVar* _n, unsigned n) {varArray[n] = _n;  }
#endif


};

/*--------------------------------------------------------------------------------------------*/
///Classe reprsentant les vecteurs de variables entires.

class IntVarArray: public NumVarArray
{
  public:
#ifndef COMPILE_FOR_PYTHON	  
   IntVarArray( Model &_modele,
                 unsigned size=0,
                 int _lb= 0,
                 int _ub= Infinity,  // ou MaxInt  voir
                 const std::string & nom = "unknow",
                 const std::string & fin = "")
    :NumVarArray( _modele,size,static_cast<float>(_lb),static_cast<float>(_ub),MuteVar::INT,nom,fin) {}
#else
    IntVarArray( Model &_modele,
                   unsigned size=0,
                   float _lb= 0,
                   float _ub= Infinity,
                   const std::string & nom = "unknow",
                   const std::string & fin = "")
    :NumVarArray( _modele,size,_lb,_ub,MuteVar::INT,nom,fin) {}
#endif
};

///Classe reprsentant les vecteurs de variables relles.
class FloatVarArray: public NumVarArray
{
  public:
    FloatVarArray( Model &_modele,
                   unsigned size=0,
                   float _lb= 0.0f,
                   float _ub= Infinity,
                   const std::string & nom = "unknow",
                 const std::string & fin = "")
    :NumVarArray( _modele,size,_lb,_ub,MuteVar::FLOAT,nom,fin) {}
};

///Classe reprsentant les vecteurs de variables boolennes.
class BoolVarArray: public NumVarArray
{
  public:
    BoolVarArray( Model &_modele,
                  unsigned size=0,
                  const std::string & nom = "unknow",
                 const std::string & fin = "")
    :NumVarArray( _modele,size,0.0f,1.0f,MuteVar::BOOL,nom,fin) {}
};

#ifdef COMPILE_FOR_PYTHON
	void export_NumVarArray();
#endif

}

#endif
