/*
	Copyright (c) 2005, Quentin Lequy and Romain Gaucher
	All rights reserved.

	Redistribution and use in source and binary forms, with or 
	without modification, are permitted provided that the following 
	conditions are met:

		* Redistributions of source code must retain the above copyright 
		  notice, this list of conditions and the following disclaimer.

		* Redistributions in binary form must reproduce the above copyright 
		  notice, this list of conditions and the following disclaimer in the 
		  documentation and/or other materials provided with the distribution.

		* Neither the name of Quentin Lequy or Romain Gaucher nor the names 
		  of its contributors may be used to endorse or promote products 
		  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
	AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef __NUMVARMATRIX_H
#define __NUMVARMARTIX_H

#include "Config.h"
#include "Constant.h"
#include "MuteVar.h"

#include <vector>
#include <string>
#include <stdexcept>

namespace Modelib {

//Pour viter les dfinitions circulaire
class Model;
class NumVar;
class NumVarArray;

//!Classe reprsentant les matrice d'inconnues
/**
  Son utilit rside dans la dclarations de familles de variables. Elles ont le mme nom  un indice prs.
*/

class NumVarMatrix
{
  protected:
  Model * modele;                   //!< Pointeur sur le modele de dfinition
  std::vector<NumVarArray *> varMatrix;    //!< Vecteur de pointeurs sur les lignes des variables
  MuteVar::VarType type;             //!< Type general du vecteur

  public:
    /**Constructeur par dfaut */
    NumVarMatrix():modele(0),type(MuteVar::FLOAT) {}

    NumVarMatrix( Model & _modele,
                 unsigned sizeI = 0,
                 unsigned sizeJ = 0,
                 float _lb= 0,
                 float _ub= Infinity,
                 MuteVar::VarType _type = MuteVar::FLOAT,
                 const std::string & nom = "unknow",
                 const std::string & milieu = "",
                 const std::string & fin = "" );

    NumVarMatrix( const NumVarMatrix & nva );
    
    NumVarMatrix( const std::vector<NumVarArray *> & nva );

    NumVarMatrix & operator= ( const NumVarMatrix & nva );

    unsigned GetNbCols() const;

    /**Geter du nombre de lignes
       @return Le nombre de lignes de la matrice de variables
    */
    unsigned GetNbRows() const
    {
        return varMatrix.size();
    }

    NumVarArray GetCol(unsigned j) const;

    NumVarArray GetRow(unsigned i) const;

    NumVar  operator() (unsigned i,unsigned j) const ;

#ifndef COMPILE_FOR_PYTHON    
	NumVar * Get(unsigned i, unsigned j);
#else
	NumVar&   Get(unsigned i, unsigned j);
#endif
    /**Geter de type
       @return Le type du vecteur de variables.
    */
    MuteVar::VarType GetType() const { return type;}

    ~NumVarMatrix();

#ifdef COMPILE_FOR_PYTHON
    NumVar&   GetItem(unsigned n) const      {return *(varArray[n]);}
    void     SetItem(NumVar* _n, unsigned n) {varArray[n] = _n;  }
#endif


};

/*--------------------------------------------------------------------------------------------*/
///Classe reprsentant les vecteurs de variables entires.

class IntVarMatrix: public NumVarMatrix
{
  public:
#ifndef COMPILE_FOR_PYTHON	  
   IntVarMatrix( Model &_modele,
                 unsigned sizeI=0,
                 unsigned sizeJ=0,
                 int _lb= 0,
                 int _ub= Infinity,  // ou MaxInt  voir
                 const std::string & nom = "unknow",
                 const std::string & milieu = "",
                 const std::string & fin = "" )
    :NumVarMatrix( _modele,sizeI,sizeJ,static_cast<float>(_lb),static_cast<float>(_ub),MuteVar::INT,nom,milieu,fin) {}
#else
    IntVarMatrix( Model &_modele,
                 unsigned sizeI=0,
                 unsigned sizeJ=0,
                 float _lb= 0,
                 float _ub= Infinity,
                 const std::string & nom = "unknow",
                 const std::string & milieu = "",
                 const std::string & fin = "" )
    :NumVarMatrix( _modele,sizeI,sizeJ,_lb,_ub,MuteVar::INT,nom,milieu,fin) {}
#endif
};

///Classe reprsentant les vecteurs de variables relles.
class FloatVarMatrix: public NumVarMatrix
{
  public:
    FloatVarMatrix( Model &_modele,
                   unsigned sizeI=0,
                   unsigned sizeJ=0,
                   float _lb= 0.0f,
                   float _ub= Infinity,
                   const std::string & nom = "unknow",
                   const std::string & milieu = "",
                   const std::string & fin = "" )
    :NumVarMatrix( _modele,sizeI,sizeJ,_lb,_ub,MuteVar::FLOAT,nom,milieu,fin) {}
};

///Classe reprsentant les vecteurs de variables boolennes.
class BoolVarMatrix: public NumVarMatrix
{
  public:
    BoolVarMatrix( Model &_modele,
                  unsigned sizeI=0,
                  unsigned sizeJ=0,
                  const std::string & nom = "unknow",
                  const std::string & milieu = "",
                  const std::string & fin = "" )
    :NumVarMatrix( _modele,sizeI,sizeJ,0.0f,1.0f,MuteVar::BOOL,nom,milieu,fin) {}
};

#ifdef COMPILE_FOR_PYTHON
	void export_NumVarMatrix();
#endif

}

#endif
