/*
	Copyright (c) 2005, Quentin Lequy and Romain Gaucher
	All rights reserved.

	Redistribution and use in source and binary forms, with or 
	without modification, are permitted provided that the following 
	conditions are met:

		* Redistributions of source code must retain the above copyright 
		  notice, this list of conditions and the following disclaimer.

		* Redistributions in binary form must reproduce the above copyright 
		  notice, this list of conditions and the following disclaimer in the 
		  documentation and/or other materials provided with the distribution.

		* Neither the name of Quentin Lequy or Romain Gaucher nor the names 
		  of its contributors may be used to endorse or promote products 
		  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
	AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "ConstraintBuilder.h"
#include <iostream>

namespace Modelib {

using namespace std;

/*-------------------------------------------------------------------------------*/

/**
  Constructeurs d'une nouvelle contraintes a partir de deux expressions et 
  d'un Operateur
  @param e1 l'expression de gauche
  @param op l'oprateur de comparaison
  @param e2 l'expression de droite
 */

ConstraintBuilder::ConstraintBuilder( const Expr & e1, ConstraintBuilder::Operateur op, const Expr & e2)
{
   expressions.push_back( e1 );
   expressions.push_back( e2 );
   operateurs.push_back( op );
   CheckIfIsBound();
}


/*-------------------------------------------------------------------------------*/

/**
  Vrifie si la contrainte est de type Bounds
  Les contraintes de type bornes sont de la forme
  - value<=variable, value<variable
  - value>=variable, value>variable
  - value<=variable<=value, value<=variable<value, value<variable<=value, value<variable<value
  - value>=variable>=value, value>=variable>value, value>variable>=value, value>variable>value
  @return vrai si la contrainte correspond au cas prcdent
*/

bool ConstraintBuilder::CheckIfIsBound()
{
  isBound = false;
  if(operateurs.size()==1 && expressions.size() == 2)
  {
    switch(operateurs.front())
    {
    	case ConstraintBuilder::INFSTRICT:
    	case ConstraintBuilder::SUPSTRICT:
    	case ConstraintBuilder::INFERIEUR:
    	case ConstraintBuilder::SUPERIEUR:
    	  isBound=(expressions.front().IsNum() && expressions.back().IsVar());
    	  isBound=isBound || (expressions.front().IsVar() && expressions.back().IsNum());
    	break;
    	default:
    	   isBound = false;
    	break;
    }
  }
  if(operateurs.size()==2 && expressions.size() == 3)
  {
      if( operateurs.front() == operateurs.back()
       || operateurs.front() == operateurs.back() + 2
       || operateurs.front() + 2 == operateurs.back() )
      {
        switch(operateurs.front())
        {
        	case ConstraintBuilder::INFERIEUR:
        	case ConstraintBuilder::SUPERIEUR:
        	case ConstraintBuilder::INFSTRICT:
        	case ConstraintBuilder::SUPSTRICT:
        	  isBound=(expressions.front().IsNum() && expressions.back().IsNum());
        	  isBound=isBound && ( (++expressions.begin())->IsVar());
        	break;
        	default:
        	   isBound = false;
        	break;
        }
      }
  }
  return isBound;
}

/*-------------------------------------------------------------------------------*/

/**
  Concatene deux constraintes en ajoutant aux listes d'une contraintes les liste
  d'expressions et d'Operateurs de l'autre afin de les concatner
  @param op le nouvel oprateur de comparaison
  @param exps une liste d'expressions provenant d'une autre contrainte
  @param ops une liste d'oprateurs provenant d'une autre contrainte
*/

void ConstraintBuilder::Add( ConstraintBuilder::Operateur op,
                        const list<Expr> & exps,
                        const list<ConstraintBuilder::Operateur> & ops)
{
    operateurs.push_back( op );
    
    for( list<Expr>::const_iterator iter = exps.begin();
                                    iter != exps.end();
                                  ++iter)
        expressions.push_back( *iter );
    
    for( list<ConstraintBuilder::Operateur>::const_iterator oper = ops.begin();
                                                   oper != ops.end();
                                                 ++oper)
        operateurs.push_back( *oper );
    CheckIfIsBound();
}

/**
   Operateur == exploitant la fonction ConstraintBuilder::Add
   @param c contrainte  concatener
   @return une rfrence sur la contrainte pour pouvoir enchaner
*/
ConstraintBuilder & ConstraintBuilder::operator== ( const ConstraintBuilder & c)
{
    Add(ConstraintBuilder::EGAL,c.expressions,c.operateurs);
    return *this;
}

/**
   Operateur != exploitant la fonction ConstraintBuilder::Add
   @param c contrainte  concatener
   @return une rfrence sur la contrainte pour pouvoir enchaner
*/
ConstraintBuilder & ConstraintBuilder::operator!= ( const ConstraintBuilder & c)
{
    Add(ConstraintBuilder::DIFFERENT,c.expressions,c.operateurs);
    return *this;
}

/**
   Operateur <= exploitant la fonction ConstraintBuilder::Add
   @param c contrainte  concatener
   @return une rfrence sur la contrainte pour pouvoir enchaner
*/
ConstraintBuilder & ConstraintBuilder::operator<= ( const ConstraintBuilder & c)
{
    Add(ConstraintBuilder::INFERIEUR,c.expressions,c.operateurs);
    return *this;
}

/**
   Operateur >= exploitant la fonction ConstraintBuilder::Add
   @param c contrainte  concatener
   @return une rfrence sur la contrainte pour pouvoir enchaner
*/
ConstraintBuilder & ConstraintBuilder::operator>= ( const ConstraintBuilder & c)
{
   Add(ConstraintBuilder::SUPERIEUR,c.expressions,c.operateurs);
   return *this;
}

/**
   Operateur < exploitant la fonction ConstraintBuilder::Add
   @param c contrainte  concatener
   @return une rfrence sur la contrainte pour pouvoir enchaner
*/
ConstraintBuilder & ConstraintBuilder::operator< ( const ConstraintBuilder & c)
{
    Add(ConstraintBuilder::INFSTRICT,c.expressions,c.operateurs);
    return *this;
}

/**
   Operateur > exploitant la fonction ConstraintBuilder::Add
   @param c contrainte  concatener
   @return une rfrence sur la contrainte pour pouvoir enchaner
*/
ConstraintBuilder & ConstraintBuilder::operator> ( const ConstraintBuilder & c)
{
   Add(ConstraintBuilder::SUPSTRICT,c.expressions,c.operateurs);
   return *this;
}


/*-----------------------------------------------------------------------------*/

/**
  Add une partie du type "== Expression"  une contrainte
  @param e l'expression  ajouter
  @return une rfrence sur la contrainte pour pouvoir enchaner
  */
ConstraintBuilder & ConstraintBuilder::operator== ( const Expr & e)
{
  expressions.push_back( e );
  operateurs.push_back( ConstraintBuilder::EGAL );
  isBound = false;
  return *this;
}

/**
  Add une partie du type "!= Expression"  une contrainte
  @param e l'expression  ajouter
  @return une rfrence sur la contrainte pour pouvoir enchaner
  */
ConstraintBuilder & ConstraintBuilder::operator!= ( const Expr & e)
{
  expressions.push_back( e );
  operateurs.push_back( ConstraintBuilder::DIFFERENT );
  isBound = false;
  return *this;
}

/**
  Add une partie du type "<= Expression"  une contrainte
  @param e l'expression  ajouter
  @return une rfrence sur la contrainte pour pouvoir enchaner
  */
ConstraintBuilder & ConstraintBuilder::operator<= ( const Expr & e)
{
  expressions.push_back( e );
  operateurs.push_back( ConstraintBuilder::INFERIEUR );
  CheckIfIsBound();
  return *this;
}

/**
  Add une partie du type ">= Expression"  une contrainte
  @param e l'expression  ajouter
  @return une rfrence sur la contrainte pour pouvoir enchaner
  */
ConstraintBuilder & ConstraintBuilder::operator>= ( const Expr & e)
{
  expressions.push_back( e );
  operateurs.push_back( ConstraintBuilder::SUPERIEUR );
  CheckIfIsBound();
  return *this;
}

/**
  Add une partie du type "< Expression"  une contrainte
  @param e l'expression  ajouter
  @return une rfrence sur la contrainte pour pouvoir enchaner
  */
ConstraintBuilder & ConstraintBuilder::operator< ( const Expr & e)
{
  expressions.push_back( e );
  operateurs.push_back( ConstraintBuilder::INFSTRICT );
  CheckIfIsBound();
  return *this;
}

/**
  Add une partie du type "> Expression"  une contrainte
  @param e l'expression  ajouter
  @return une rfrence sur la contrainte pour pouvoir enchaner
  */
ConstraintBuilder & ConstraintBuilder::operator> ( const Expr & e)
{
  expressions.push_back( e );
  operateurs.push_back( ConstraintBuilder::SUPSTRICT );
  CheckIfIsBound();
  return *this;
}


/*---------------------------------------------------------------------------*/

/**
  Construction de contraintes a partir de deux expressions
  @param e1 expression de gauche
  @param e2 expression de droite
  @return l'Expr e1 == e2
*/
ConstraintBuilder operator== ( const Expr & e1, const Expr & e2)
{
   return ConstraintBuilder( e1, ConstraintBuilder::EGAL, e2 );
}

/**
  Construction de contraintes a partir de deux expressions
  @param e1 expression de gauche
  @param e2 expression de droite
  @return l'Expr e1 != e2
*/
ConstraintBuilder operator!= ( const Expr & e1, const Expr & e2)
{
   return ConstraintBuilder( e1, ConstraintBuilder::DIFFERENT, e2 );
}

/**
  Construction de contraintes a partir de deux expressions
  @param e1 expression de gauche
  @param e2 expression de droite
  @return l'Expr e1 <= e2
*/
ConstraintBuilder operator<= ( const Expr & e1, const Expr & e2 )
{
   return ConstraintBuilder( e1, ConstraintBuilder::INFERIEUR, e2 );
}

/**
  Construction de contraintes a partir de deux expressions
  @param e1 expression de gauche
  @param e2 expression de droite
  @return l'Expr e1 >= e2
*/
ConstraintBuilder operator>= ( const Expr & e1, const Expr & e2)
{
   return ConstraintBuilder( e1, ConstraintBuilder::SUPERIEUR, e2 );
}

/**
  Construction de contraintes a partir de deux expressions
  @param e1 expression de gauche
  @param e2 expression de droite
  @return l'Expr e1 < e2
*/
ConstraintBuilder operator< ( const Expr & e1, const Expr & e2 )
{
   return ConstraintBuilder( e1, ConstraintBuilder::INFSTRICT, e2 );
}

/**
  Construction de contraintes a partir de deux expressions
  @param e1 expression de gauche
  @param e2 expression de droite
  @return l'Expr e1 > e2
*/
ConstraintBuilder operator> ( const Expr & e1, const Expr & e2)
{
   return ConstraintBuilder( e1, ConstraintBuilder::SUPSTRICT, e2 );
}

/*------------------------------------------------------------------------------*/

/**
  Operateur de surcharge de flux <<
  @param flux  flux  surcharger
  @param c  la contrainte  afficher
  @return Le flux surcharg
*/
std::ostream & operator<< (std::ostream & flux, const ConstraintBuilder & c)
{
    string op[] = {  "==", "!=", "<=", ">=", " < ", " > " };
    list<Expr>::const_iterator iter = c.expressions.begin();
    list<ConstraintBuilder::Operateur>::const_iterator oper = c.operateurs.begin();

    while( iter != c.expressions.end() )
    {
    	flux << *iter;
    	if(oper != c.operateurs.end())
        {
        	flux << op[*oper];
        	++oper;
        }
    	++iter;
    }
    return flux;
}


}
