/*
	Copyright (c) 2005, Quentin Lequy and Romain Gaucher
	All rights reserved.

	Redistribution and use in source and binary forms, with or 
	without modification, are permitted provided that the following 
	conditions are met:

		* Redistributions of source code must retain the above copyright 
		  notice, this list of conditions and the following disclaimer.

		* Redistributions in binary form must reproduce the above copyright 
		  notice, this list of conditions and the following disclaimer in the 
		  documentation and/or other materials provided with the distribution.

		* Neither the name of Quentin Lequy or Romain Gaucher nor the names 
		  of its contributors may be used to endorse or promote products 
		  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
	AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "Model.h"
#include "FichierLP.h"
#include "FichierMPS.h"
#include <list>
#include <iostream>
#include <sstream>
#include <cmath>
#include <stdexcept>

namespace Modelib {

using namespace std;

/**Constructeur par copie*/
Model::Model(const Model &m)
{
    *this = m;
}

/**operateur de copie*/
Model & Model::operator=(const Model &m)
{
    if( this != &m)
    {
      hacheVarId=m.hacheVarId;
      unknowMap=m.unknowMap;
      A.matrix = m.A.matrix;
      A.nbLignes = m.A.nbLignes;
      A.nbCols = m.A.nbCols;
      constraintMap = m.constraintMap;
  
      isMax = m.isMax;
      isRelaxed = m.isRelaxed;
      isFract = m.isFract;
      isOptimal = m.isOptimal;
      isSolved = m.isSolved;
      MIPVar = m.MIPVar;
  
      objectif = m.objectif;
      objectifActivity = m.objectifActivity;
      objectifName = m.objectifName;
      problemName = m.problemName;
      
      solverAPI = m.solverAPI;
    }
    return *this;
}

/*-------------------------------------------------------------------------------------------------------*/
/**Recupere le coefficient du multiplicatuer de Lagrange asssoci  la contrainte
 @param id li'dentifiant de contrainte
 */
float Model::GetLagrangianParamValue( VarId id ) const
{
  std::map<VarId,InternalConstraint>::const_iterator trouve = constraintMap.find(id);
  if(trouve!=constraintMap.end())
     return trouve->second.lagrangianValue;
  else
     cerr << "Warning in Model::GetLagrangianParamValue : undefined id " << id<< endl;
  return 0;
}


/**Definit le coefficient du multiplicateur de Lagrange asssoci  la contrainte
 @param id li'dentifiant de contrainte
 */
void Model::SetLagrangianParamValue(VarId id, float value)
{
  std::map<VarId,InternalConstraint>::iterator trouve = constraintMap.find(id);
  if( trouve!=constraintMap.end())
  {
    //selon le type de contrainte les domaines de dfinition du multiplicateur varient
    switch(trouve->second.type)
    {
     	case InternalConstraint::INFSTRICT :
     	case InternalConstraint::INF :
        //Pour les contraintes de type infrieur, le multiplicateur est positif
        if(value < 0)
     	  {
     	  	 value = 0;
           cerr << "Warning in Model::SetLagrangianParamValue : constraint " << id
                  << " lagrangian value is out of his definition area" << endl;
     	  }
     	break;

     	case InternalConstraint::SUPSTRICT :
     	case InternalConstraint::SUP :
        //Pour les contraintes de type infrieur, le multiplicateur est positif
        if(value > 0)
     	  {
     	  	 value = 0;
             cerr << "Warning in Model::SetLagrangianParamValue : constraint " << id
                  << " lagrangian value is out of his definition area" << endl;
     	  }
     	break;

     	case InternalConstraint::EGAL :
        //Pour les contraintes de type egal, le multiplicateur est rel
     	break;

     	default:
           cerr << "Warning in Model::SetLagrangianParamValue : wrong InternalConstraint::Type" << endl;
     	break;
     }
     trouve->second.lagrangianValue = value;
  }
  else
     cerr << "Warning in Model::SetLagrangianParamValue : undefined id " << id << endl;
}


/*--------------------------------------------------------------------------------------------------------*/
/**
  AddVar permet la cration d'une nouvelle variable referenc dans le modele. \n
  Seule la classe NumVar appelle cette fonction. \n
  Elle est d'ailleurs dclare en friend spcialement pour pouvoir utiliser cette fonction\n
  @return l'identifiant attribu  la variable
 */

VarId Model::AddVar( const MuteVar & muteVar, const std::string & _nom )
{
  string tmpnom;
  //futur nom de la variable
  //si la variable n'a pas de nom dfini
  if( _nom == "unknow" )
  {
    char * nomId = new char[30];
    //on recupere son futur identifiant dans un buffer de caractre
    sprintf(nomId,"%d",hacheVarId);
    tmpnom = _nom + nomId ;
    //le futur nom de la variable sera donc unknow{id}
    delete [] nomId;
    //on libere le buffer
  }
  else
    tmpnom = _nom;
  //sinon on son futur nom est le nom donner par l'utilisateur
  unknowMap[hacheVarId]=Paire(muteVar,tmpnom);
  //on insere la variable dans le map
  if( muteVar.GetType() != MuteVar::FLOAT)
    MIPVar++;
  //si la variable n'est pas continues on augmente le compteur des variables MIP
  return hacheVarId++;
  //on retourne l'identifiant puis on l'augmente
}

/*--------------------------------------------------------------------------------------------------------*/
/**
  Geter de la variable associe a l'identifiant pass en argument
  @param varId l'identifiant
  @return la variable
 */
MuteVar Model::GetMuteVar( VarId varId) const
{
  std::map<VarId,Paire>::const_iterator trouve = unknowMap.find(varId);
  if(trouve!=unknowMap.end())
     return trouve->second.variable;
  else
     cerr << "Warning in Model::GetMuteVar : undefined id " << varId << endl;
  return MuteVar();
}


/**
  Geter de la variable associe a l'identifiant pass en argument
  @param varId l'identifiant
  @return la variable  sous le formar NumVar
 */
NumVar Model::GetNumVar( VarId varId) const
{
  return NumVar(this,varId);
}

/**
  Geter du nom de la variable associe  l'identifiant pass en argument
   @param varId l'identifiant
   @return le nom de la variable
*/
std::string Model::GetName( VarId varId ) const
{
  std::map<VarId,Paire>::const_iterator trouve = unknowMap.find(varId);
  if(trouve!=unknowMap.end())
     return trouve->second.nom;
  else
     cerr << "Warning in Model::GetName : undefined id " << varId << endl;
  return defaultGetName;
}

/**
  Seter du nom de la variable associe  l'identifiant pass en argument
   @param varId l'identifiant
   @param le nom de la variable
*/
void Model::SetName( VarId varId, const std::string & name )
{
  std::map<VarId,Paire>::iterator trouve = unknowMap.find(varId);
  if(trouve!=unknowMap.end())
     trouve->second.nom = name;
  else
     cerr << "Warning in Model::GetName : undefined id " << varId << endl;
}

VarId Model::GetVarId(const std::string & name) const
{
   std::map<VarId,Paire>::const_iterator trouve = unknowMap.begin();
   VarId retour = 0;
   //Pour toutes les variables
   for(;trouve!=unknowMap.end();++trouve)
     //si on trouve le nom
     if(trouve->second.nom == name)
     {
       retour = trouve->first;
       //on rcupre l'identifiant et on arrte la boucle
       break;
     }
   if(retour == 0)
     cerr << "Warning in Model::GetVarId : undefined name " << name << endl;
   return retour;

}


/*--------------------------------------------------------------------------------------------------------*/
/**
  Geter de la borne infrieure de la variable associe  l'identifiant pass en argument
  @param varId l'identifiant
  @return la borne infrieure
 */
float Model::GetMuteVarLB( VarId varId) const
{
  std::map<VarId,Paire>::const_iterator trouve = unknowMap.find(varId);
  if(trouve!=unknowMap.end())
     return (trouve->second.variable).GetLowerBound();
  else
     cerr << "Warning in Model::GetMuteVarLB : undefined id " << varId << endl;
  return 0.0;
}

/**
  Seter de la borne infrieure de la variable associe  l'identifiant pass en argument
  @param varId l'identifiant
  @param _value la nouvelle value de la borne infrieure
 */
void Model::SetMuteVarLB( VarId varId, float _value)
{
  std::map<VarId,Paire>::iterator trouve = unknowMap.find(varId);
  if(trouve!=unknowMap.end())
     (trouve->second.variable).SetLowerBound( _value);
  else
      cerr << "Warning in Model::SetMuteVarLB : undefined id " << varId << endl;
 return;
}

/*--------------------------------------------------------------------------------------------------------*/
/**
  Geter de la borne suprieure de la variable associe  l'identifiant pass en argument
  @param varId l'identifiant
  @return la borne suprieure
 */
float Model::GetMuteVarUB( VarId varId) const
{
  std::map<VarId,Paire>::const_iterator trouve = unknowMap.find(varId);
  if(trouve!=unknowMap.end())
     return  (trouve->second.variable).GetUpperBound();
  else
      cerr << "Warning in Model::GetMuteVarUB : undefined id " << varId << endl;
  return Infinity;
}


/**
  Seter de la borne suprieure de la variable associe  l'identifiant pass en argument
  @param varId l'identifiant
  @param _value la nouvelle value de la borne suprieure
 */
void Model::SetMuteVarUB( VarId varId, float _value)
{
  std::map<VarId,Paire>::iterator trouve = unknowMap.find(varId);
  if(trouve!=unknowMap.end())
     (trouve->second.variable).SetUpperBound( _value);
  else
      cerr << "Warning in Model::SetMuteVarUB : undefined id " << varId << endl;
  return;
}

/*--------------------------------------------------------------------------------------------------------*/
/**
  Seter du caractre strict de borne infrieure de la variable
  associe  l'identifiant pass en argument
  @param varId l'identifiant
  @param _value la nouvelle value du caractre strict de la borne infrieure
 */
void Model::SetMuteVarLBStrict( VarId varId, bool _value)
{
  std::map<VarId,Paire>::iterator trouve = unknowMap.find(varId);
  if(trouve!=unknowMap.end())
     (trouve->second.variable).SetIsLBStrict( _value);
  else
      cerr << "Warning in Model::SetMuteVarLBStrict : undefined id " << varId << endl;
  return;
}

/**
  Seter du caractre strict de borne suprieure de la variable
  associe  l'identifiant pass en argument
  @param varId l'identifiant
  @param _value la nouvelle value du caractre strict de la borne suprieure
 */
void Model::SetMuteVarUBStrict( VarId varId, bool _value)
{
  std::map<VarId,Paire>::iterator trouve = unknowMap.find(varId);
  if(trouve!=unknowMap.end())
     (trouve->second.variable).SetIsUBStrict( _value);
  else
      cerr << "Warning in Model::SetMuteVarUBStrict : undefined id " << varId << endl;
  return;
}

/*--------------------------------------------------------------------------------------------------------*/
/**
  Geter du type  de la variable associe  l'identifiant pass en argument
  @param varId l'identifiant
  @return le type de la variable (sous le format MuteVar::VarType)
 */
MuteVar::VarType Model::GetType( VarId varId) const
{
  std::map<VarId,Paire>::const_iterator trouve = unknowMap.find(varId);
  if(trouve!=unknowMap.end())
     return (trouve->second.variable).GetType();
  else
     cerr << "Warning in Model::GetType : undefined id " << varId << endl;
  return MuteVar::FLOAT;
}

/**
  Seter du type  de la variable associe  l'identifiant pass en argument
  @param varId l'identifiant
  @param le nouveau type de la variable (sous le format MuteVar::VarType)
 */                               
void Model::SetType(  VarId varId, MuteVar::VarType _type)
{
  std::map<VarId,Paire>::iterator trouve = unknowMap.find(varId);
  if(trouve!=unknowMap.end())
  {
  	if( trouve->second.variable.GetType()!=MuteVar::FLOAT 
      &&  _type == MuteVar::FLOAT)
  	     MIPVar++;
  	if( trouve->second.variable.GetType()==MuteVar::FLOAT
       &&  _type != MuteVar::FLOAT)
  	     MIPVar--;
  	(trouve->second.variable).SetType( _type);
  }
  else
     cerr << "Warning in Model::SetType : undefined id " << varId << endl;
  return;
}

/*--------------------------------------------------------------------------------------------------------*/
/**Dfinit la fonction  maximiser
   @param expr l'expression  maximiser
   @param nom le nom de l'expression  maximiser
 */
void Model::Maximize(const Expr & expr , const std::string & name )
{
   objectif = expr;
   objectif.Simplify();
   isMax = true;
   objectifName = name;
}

/**Dfinit la fonction  minimiser
   @param expr l'expression  minimiser
   @param nom le nom de l'expression  maximiser
 */
void Model::Minimize(const Expr & expr, const std::string & name )
{
   objectif = expr;
   objectif.Simplify();
   isMax = false;
   objectifName = name;
}

/*--------------------------------------------------------------------------------------------------------*/
/**
  Fonction permettant d'ajouter la partie linaire d'une contrainte au modele
  Il suffit juste de modifier le coefficient de la matrice adquat pour chaque variable, \n
   savoir celui d'indice (l'identifiant de variable, numro de contrainte)
  @param expr  l'expression  ajouter au modle
  @param A  la matrice qui correspond au type de contrainte d'ou provient l'expression
 */
VarId Model::AddExpr( Expr & expr , MatriceCreuse<float> & A )
{
   std::list<ExprNode>::iterator iter;
   unsigned p = A.GetNbLignes()+1;
   for( iter = expr.nodeList.begin(); iter != expr.nodeList.end(); ++iter)
   {
      float coef = iter->GetCoefficient().GetValue();
      NumVar varTmp = iter->GetVariable();
      VarId varId = (varTmp.GetModel()?varTmp.GetVarId():0);
      if(varId && unknowMap.find(varId) != unknowMap.end())
      {
      	A.Set(varId,p,coef);
      }
      else
      {
        if(varId)
            cerr << "Warning in Model::AddExpr : undefined id " << varId << endl;
      }
    }
  return hacheConstraintId++;
}

/*---------------------------------------------------------------------------------------*/

/**
  Add une contrainte au modele
  @param c  la contrainte  ajouter
  @param nom le nom de la contrainte
  @return le type de matrice dans laquelle a t insrer la contrainte
 */
//Attention: Contraint a mettre ici
Constraint Model::Add( const ConstraintBuilder & c , const std::string & nom)
{
  Constraint retour;
  if( c.IsBound() )  //La contrainte est une borne
  {
       NumVar var = (++c.expressions.begin())->nodeList.front().GetVariable();

       //La borne est de type: {value} {Operateur} {Variable} ou {Variable} {Operateur} {value}
       if(c.operateurs.size()==1)
       {

         float bound;
         bool prems = ( c.expressions.back().nodeList.front().GetVariable().GetModel() == 0 );
         //booleen servant a savoir si la variable est en premier ou pas
         //si elle est en premier
         if(prems)
            bound = c.expressions.back().nodeList.front().GetCoefficient().GetValue();
            //on rcupere le coefficient de la seconde partie de l'expression qui est la borne suprieure
         else
            bound = c.expressions.front().nodeList.front().GetCoefficient().GetValue();
            //on rcupere le coefficient de la premire partie de l'expression qui est la borne infrieure
         list<ConstraintBuilder::Operateur>::const_iterator op = c.operateurs.begin();
         //En cas de contraintes stricte il faut modifi le caractere des borne
         //suivant le type de contraintes
         switch( *op )
         {
           case ConstraintBuilder::INFSTRICT: //cas similaire  INFERIEUR
              if(prems)
                 var.SetUpperBoundStrict( true );
              else
                 var.SetLowerBoundStrict( true );
           case ConstraintBuilder::INFERIEUR:
              if(prems)
                 var.SetUpperBound( bound );
              else
                 var.SetLowerBound( bound );
           break;
           case ConstraintBuilder::SUPSTRICT: //cas similaire  SUPERIEUR
              if(!prems)
                 var.SetUpperBoundStrict( true );
              else
                 var.SetLowerBoundStrict( true );
           case ConstraintBuilder::SUPERIEUR:
             if(!prems)
                 var.SetUpperBound( bound );
              else
                 var.SetLowerBound( bound );
           break;
           default:
                 cerr << "Warning in Model::Add : wrong ConstraintBuilder::Operator for single constraint" << endl;
           break;
         }
       }

       //La Contrainte est de la forme {value} {Operateur} {variable} {Operateur} {value}
       if(c.operateurs.size()==2)
       {
         float lrB = c.expressions.front().nodeList.front().GetCoefficient().GetValue();
         //on rcupere la borne infrieure
         float urB = c.expressions.back().nodeList.front().GetCoefficient().GetValue();
         //on rcupere la borne suprieure
         list<ConstraintBuilder::Operateur>::const_iterator op = c.operateurs.begin();
         //on rcupere l'oprateur de comparaison
         float swap;
         //En cas de contraintes superieure il faut inverser les bornes
         switch( *op )
         {
           case ConstraintBuilder::INFSTRICT:
             var.SetLowerBoundStrict( true );
           break;
           case ConstraintBuilder::INFERIEUR:
           break;
           case ConstraintBuilder::SUPSTRICT:
             var.SetUpperBoundStrict( true );
           case ConstraintBuilder::SUPERIEUR:
             swap = lrB;
             lrB = urB;
             urB = swap;
           break;
           default:
             cerr << "Warning in Model::Add : wrong ConstraintBuilder::Operator for double constraint" << endl;
           break;
         }
         ++op;
         //on rcupere l'oprateur de comparaison suivant
         if( *op == ConstraintBuilder::INFSTRICT)
           var.SetUpperBoundStrict( true );
         if( *op == ConstraintBuilder::SUPSTRICT) 
           var.SetLowerBoundStrict( true );
         var.SetLowerBound( lrB );
         var.SetUpperBound( urB );
       }
  }
  //sinon c'est une contrainte generique
  else
  {
    if( c.operateurs.size() >=1 && c.operateurs.size() + 1 == c.expressions.size()  )
    {
    	//La contrainte est valide
      list<Expr>::const_iterator exp = c.expressions.begin();
    	list<Expr>::const_iterator exp2 = ++c.expressions.begin();
    	list<ConstraintBuilder::Operateur>::const_iterator op = c.operateurs.begin();

    	//Decompostion de la suite de contraintes
        while( exp2!=c.expressions.end() )
        {
          Expr tmp; //expressions temporaire qui va aider  sparer la chaine de contraintes
          ostringstream id;
          if(c.operateurs.size()!=1 || nom.length()== 0 )
             if( nom.length()== 0 )
                id << "__c" << A.nbLignes+1;
             else
                id << nom << A.nbLignes+1;
          else
              id << nom;
          string nom2 = id.str();

          tmp = *exp - *exp2;
          //la partie linaire de la contrainte est le premier membre moins le second membre
          tmp.Simplify();
          //on simplifie

          VarId p;
          float coef = -(tmp.nodeList.front().GetVariable().GetModel()?0.0f:tmp.nodeList.front().GetCoefficient().GetValue());
          //on recupere le coefficient qui se trouve en premiere position dans l'expression
          //suivant le type de contrainte
          //on ajoute la partie linaire  la matrice
          //et on cr la representation interne de la contrainte dans le vecteur de contrainte
          switch( *op )
          {
          	case ConstraintBuilder::INFERIEUR:
               p = AddExpr( tmp, A );
               constraintMap[p] = InternalConstraint(InternalConstraint::INF,coef,0.0,0.0,nom2,false,false);
          	break;

            case ConstraintBuilder::SUPERIEUR:
               p = AddExpr( tmp, A );
               constraintMap[p] = InternalConstraint(InternalConstraint::SUP,coef,0.0,0.0,nom2,false,false);
          	break;

          	case ConstraintBuilder::EGAL:
               p = AddExpr( tmp, A );
               constraintMap[p] = InternalConstraint(InternalConstraint::EGAL,coef,0.0,0.0,nom2,false,false);
          	break;

          	case ConstraintBuilder::INFSTRICT:
               p = AddExpr( tmp, A );
               constraintMap[p] = InternalConstraint(InternalConstraint::INFSTRICT,coef,0.0,0.0,nom2,false,false);
          	break;

            case ConstraintBuilder::SUPSTRICT:
               p = AddExpr( tmp, A );
               constraintMap[p] = InternalConstraint(InternalConstraint::SUPSTRICT,coef,0.0,0.0,nom2,false,false);
          	break;

            default:
              cerr << "Warning in Model::Add : wrong ConstraintBuilder::Operator" << endl;
         	  break;
          }
           ++exp2;++exp;++op;
           //on passe   la contrainte suivante 
           retour = Constraint(*this,p);
    	}
    }
    else
      cerr << "Warning in Model::Add : the ConstraintBuilder is bad structured" << endl;
  }
  return retour;
}


/*---------------------------------------------------------------------------------------*/

/**
  Sortie graphique sous forme brute
  @param flux  le flux de destination
  @param newline le chaine a mettre en bout de ligne
 */
std::ostream & Model::Out( std::ostream & flux, const std::string & newline )
{
    flux << (isMax?"Maximize: ":"Minimize") << objectif << newline << endl << endl;
    flux << "A =" << newline << endl;
    A.Out();
    flux << "Matrice de taille: " << A.GetNbLignes() 
        << "x" << A.GetNbCols() << newline << endl;
    return flux;
}


/*---------------------------------------------------------------------------------------*/

/**
  Sortie graphique sous forme de flux dans le format LP
  @param o le flux  surcharger
  @return le flux surcharg
  */

std::ostream& Model::SurchargeFlux(std::ostream& flux) const
{
    flux << "\\" << problemName << endl;

    if( isMax)
        flux <<  "Maximize " << endl;
    else
        flux <<  "Minimize " << endl;
    flux << objectifName << (objectifName.length()?": ":" ") << objectif << endl;

    flux << endl << "Subject to" << endl;

    string op[]  = { "<=",">=","="," > "," < "};


    VarId p = A.nbLignes+1;
    bool normal = false;

    std::map<VarId,Paire>::const_iterator variable;
    std::map<VarId,InternalConstraint>::const_iterator constraint = constraintMap.end();
    std::map<VarId,std::string>::const_iterator comment = comments.begin();

    for(map< Id,float,Compare<float> >::const_iterator iter  = A.matrix.begin();
                                                       iter != A.matrix.end();
                                                     ++iter)
    {
        unsigned i = iter->first.i; //colonne
        unsigned j = iter->first.j; //ligne attention au dcalage d'indice avec tB

        variable  = unknowMap.find(VarId(i));

        if(j != p)  //nouvelle contrainte
        {
            if(constraint != constraintMap.end())
                flux << op[constraint->second.type] << constraint->second.b << endl;

            constraint = constraintMap.find(j);
            p = constraint->first;

            while( comment != comments.end() && comment->first <= p)
            {
                flux << "\\" << comment->second << endl;
                ++comment;
            }
            if( constraint->second.name.length())
                flux << constraint->second.name << ": ";

            normal = false;
        }

        float coeff = iter->second;

        if( normal )
        {
            if(coeff != 0.0f)
            {
                if(coeff != 1.0f && coeff != -1.0f )
                        if( coeff > 0 )
	                           flux  << "+" << coeff << variable->second.nom;
                        else
	                           flux << coeff << variable->second.nom;
                else
                        if( coeff == 1.0f )
                               flux  << "+" << variable->second.nom;
                        else
	                           flux  << "-" << variable->second.nom;
            }
        }
        else
        {
            if(coeff != 0.0f)
            {
                if(coeff != 1.0f && coeff != -1.0f )
                    flux  << coeff << variable->second.nom;
                else
                    flux << (coeff == 1.0?"":"-") << variable->second.nom;
                normal = true;
            }
        }

    }
    if(constraint != constraintMap.end())
        flux << op[constraint->second.type] << constraint->second.b << endl;

    while( comment != comments.end() )
    {
        flux << "\\" << comment->second << endl;
        ++comment;
    }


    bool bounds = false;
    for(std::map<VarId,Paire>::const_iterator variable  = unknowMap.begin();
                                              variable != unknowMap.end();
                                            ++variable)
    {
  	    bool lbs = variable->second.variable.IsLowerBoundStrict();
   	    bool ubs = variable->second.variable.IsUpperBoundStrict();
   	    float lb = variable->second.variable.GetLowerBound();
   	    float ub = variable->second.variable.GetUpperBound();

        if( variable->second.variable.GetType() != MuteVar::BOOL)
        {
            if( (lb != 0 || ub != Infinity) && bounds == false  )
            {
               flux << endl << "Bounds" << endl;
            	 bounds = true;
    	      }
            if( bounds == true )
            {
                if(lb == -Infinity && ub == Infinity)
                    flux <<  variable->second.nom << " Free";
                else
                {
        	       if(lb!=0.0)
        	       {
        	           if( lb != -Infinity )
                            flux << lb << (lbs?" < ":"<=");
                        else
                            flux << "-Inf" << " < ";
                    }

                    if(ub!=Infinity)
                        flux << variable->second.nom << (ubs?" < ":" <= ") << ub;
                    else
                        if(lb!=0.0)
                            flux << variable->second.nom;
                }
                if( lb!=0.0 || ub!=Infinity )
                  flux << endl;
            }
        }
    }

    bool generals = false;
    for(std::map<VarId,Paire>::const_iterator variable  = unknowMap.begin();
                                            variable != unknowMap.end();
                                          ++variable)
    {
        if(variable->second.variable.GetType() == MuteVar::INT)
        {
            if(generals)
                flux << variable->second.nom << endl;
            else
            {
                flux << endl  << "Generals" << endl
                     << variable->second.nom << endl;
        	    generals = true;
            }
        }
    }


    bool binaries = false;
    for(std::map<VarId,Paire>::const_iterator variable  = unknowMap.begin();
                                            variable != unknowMap.end();
                                          ++variable)
    {
        if(variable->second.variable.GetType() == MuteVar::BOOL)
        {
            if(binaries == true)
                flux << variable->second.nom << endl;
            else
            {
                flux << endl << endl << "Binaries" << endl
                     << variable->second.nom << endl;
        	    binaries = true;
            }
        }
    }
  
    flux<< endl << endl << "End" << endl;

    return flux;
}

/**
  Sortie graphique des dimensions du problme
  */
std::ostream & Model::OutDimensions( std::ostream & flux, const std::string & newline)
{
   flux << "Tailles des matrices:" << newline << endl;
   flux << "   Matrice des ingalits: " << A.GetNbLignes()
        << "x" << A.GetNbCols() << newline << endl
        << "       taux d'occupation: "
        << (A.GetNbLignes()!=0.0f?A.matrix.size()*100.0/(1.0*A.nbLignes*A.nbCols):0)
        << "% "   << endl << newline;
   flux << "Nombre de Variables:  " << unknowMap.size() << newline << endl;
   flux << "Nombre de Contraintes:  " << constraintMap.size() << newline << endl;
   flux << "Taille de la fonction objectif:  " << objectif.Size() << newline << endl;
   return flux;
}

/*---------------------------------------------------------------------------------------*/

/**
  Sortie graphique sous forme de flux dans le format LP du probleme relax
  @param o le flux  surcharger
  @return le flux surcharg
  */
std::ostream& Model::SurchargeFluxRelaxed(std::ostream& flux) const
{

  flux << "\\" << problemName << endl;

/*  list<ExprNode>::const_iterator src = objectif.nodeList.begin();

  for(;src!=objectif.nodeList.end();++src)
    cout << "+" << src->GetCoefficient() << "*" << src->GetVariable().GetName()
         << "*" << src->GetLagrangianParam().GetValue();
    cout << endl;
*/
    if( isMax)
        flux <<  "Maximize " << endl;
    else
        flux <<  "Minimize " << endl;
    flux << "\\Constante  ajouter: " << objectif.GetConstant( true ) << endl;
    flux << objectifName << (objectifName.length()?": ":" ") << objectif << endl;

    flux << endl << "Subject to" << endl;

    string op[]  = { "<=",">=","="," > "," < "};

    VarId p = A.nbLignes+1;
    bool normal = false;

    map<VarId,Paire>::const_iterator variable;
    map<VarId,InternalConstraint>::const_iterator constraint = constraintMap.end();
    map<VarId,std::string>::const_iterator comment = comments.begin();

    for(map< Id,float,Compare<float> >::const_iterator iter  = A.matrix.begin();
                                                       iter != A.matrix.end();
                                                     ++iter)
    {
        unsigned i = iter->first.i; //colonne
        unsigned j = iter->first.j; //ligne

        variable  = unknowMap.find(VarId(i));

        if(j != p)  //nouvelle contrainte
        {
             if(  constraint != constraintMap.end()
             && !constraint->second.isLagrangianParamActive )
                flux << op[constraint->second.type] << constraint->second.b << endl;

            constraint = constraintMap.find(j);
            p = constraint->first;

            while( comment != comments.end() && comment->first <= p)
            {
                flux << "\\" << comment->second << endl;
                ++comment;
            }

            if( constraint->second.name.length() && !constraint->second.isLagrangianParamActive)
                flux << constraint->second.name << ": ";

            normal = false;
        }

        float coeff = iter->second;

        if( !constraint->second.isLagrangianParamActive )
        {
            if( normal )
            {
                if(coeff != 0.0f)
                {
                    if(coeff != 1.0f && coeff != -1.0f )
                            if( coeff > 0 )
    	                           flux  << "+" << coeff << variable->second.nom;
                            else
    	                           flux << coeff << variable->second.nom;
                    else
                            if( coeff == 1.0f )
                                   flux  << "+" << variable->second.nom;
                            else
    	                           flux  << "-" << variable->second.nom;
                }
            }
            else
            {
                if(coeff != 0.0f)
                {
                    if(coeff != 1.0f && coeff != -1.0f )
                        flux  << coeff << variable->second.nom;
                    else
                        flux << (coeff == 1.0?"":"-") << variable->second.nom;
                    normal = true;
                }
            }
        }

    }
    if(  constraint != constraintMap.end()
     && !constraint->second.isLagrangianParamActive )
        flux << op[constraint->second.type] << constraint->second.b << endl;

    while( comment != comments.end() )
    {
        flux << "\\" << comment->second << endl;
        ++comment;
    }


  bool bounds = false;
  for(map<VarId,Paire>::const_iterator variable  = unknowMap.begin();
                                       variable != unknowMap.end();
                                     ++variable)
  {
  	bool lbs = variable->second.variable.IsLowerBoundStrict();
   	bool ubs = variable->second.variable.IsUpperBoundStrict();
   	float lb = variable->second.variable.GetLowerBound();
   	float ub = variable->second.variable.GetUpperBound();

    if( variable->second.variable.GetType() != MuteVar::BOOL)
    {
      if( (lb != 0 || ub != Infinity) && bounds == false )
      {
        flux << endl << "Bounds" << endl;
      	bounds = true;
      }

      if( bounds == true )
      {
        if(lb == -Infinity && ub == Infinity)
            flux <<  variable->second.nom << " Free";
        else
        {
        	if(lb!=0.0)
        	{
        	   if( lb != -Infinity )
                 flux << lb << (lbs?" < ":"<=");
               else
                 flux << "-Inf" << " < ";
            }

            if(ub!=Infinity)
               flux << variable->second.nom << (ubs?" < ":"<=") << ub;
            else
               if(lb!=0.0)
                 flux << variable->second.nom;
        }
        if( lb!=0.0 || ub!=Infinity )
          flux << endl;
      }
    }
  }

  bool generals = false;
  for(map<VarId,Paire>::const_iterator variable  = unknowMap.begin();
                                       variable != unknowMap.end();
                                     ++variable)
  {
     if(variable->second.variable.GetType() == MuteVar::INT && !variable->second.variable.IsRelaxed())
     {
     	if( generals == true )
          flux << variable->second.nom << endl;
        else
        {
        	flux << endl  << "Generals" << endl
                 << variable->second.nom << endl;
        	generals = true;
        }
     }
  }


  bool binaries = false;
  for(map<VarId,Paire>::const_iterator variable  = unknowMap.begin();
                                       variable != unknowMap.end();
                                     ++variable)
  {
     if(variable->second.variable.GetType() == MuteVar::BOOL && !variable->second.variable.IsRelaxed())
     {
     	if(binaries == true )
          flux << variable->second.nom << endl;
        else
        {
            flux << endl << endl << "Binaries" << endl
                 << variable->second.nom << endl;
        	binaries = true;
        }
     }
  }

  flux<< endl << endl << "End" << endl;

  return flux;
}

/**Surcharge de l'operateur de flux
  @param flux le flux  surcharger
  @param m le modele  afficher
  @return Le flux surcharg
 */
std::ostream& operator<<(std::ostream& flux,const Model& m)
{
  if( m.IsRelaxed() )
     return m.SurchargeFluxRelaxed(flux);
  else
     return m.SurchargeFlux(flux);

}


/*-----------------------------------------------------------------------------------------------*/

/**Surcharge de l'operateur de flux
  @param flux le flux  surcharger
  @param m le modele  afficher
  @return Le flux surcharg
 */
std::ostream & Model::SurchargeMPSFlux(std::ostream & flux,bool verbose) const
{
  string pbName(problemName,0,8);
  if( problemName.length() >= 8 )
  {
    cerr << "Warning in Model::SurchargeMPSFlux : the probleme name is too long for MPS" << endl;
  }
  string::size_type endline = pbName.find('\n');
  if( endline != string::npos && verbose==true )
  {
    cerr << "Warning in Model::SurchargeMPSFlux : the probleme name is reduced to a single line" << endl;
  }
  pbName = pbName.substr(0,endline);
  flux << "NAME          " << pbName << endl;
  
  string objName(objectifName,0,8);
  if( objectifName.length() > 8 && verbose==true )
  {
    cerr << "Warning in Model::SurchargeMPSFlux : the objectif name is too long for MPS format" << endl;
  }
  
  flux << "ROWS" << endl << " N  " << objName << endl;
  map<VarId, InternalConstraint>::const_iterator constraint;
  string strType[5] = {" L  "," G  "," E  "," L  "," G  "};
  for( constraint = constraintMap.begin(); constraint != constraintMap.end(); ++constraint )
  {
    string strConstraintName(constraint->second.name);
    if( strConstraintName.length() > 8 )
    {
      if( verbose==true )
        cerr << "Warning in Model::SurchargeMPSFlux : The constraint " << strConstraintName << " has a too long name for the MPS format. This name will be replaced by ";
      strConstraintName = Nom("_",constraint->first)();
      if( verbose==true )
        cerr << strConstraintName << "." << endl;
   }
     flux << strType[int(constraint->second.type)] << strConstraintName << endl;
  }
  
  map< VarId, ostringstream * > tempM;
  map< VarId, ostringstream *>::iterator iterStr;
  list< ostringstream * > generals[2];
  list< ostringstream * >::iterator iterlistStr;
  for(map<VarId,Paire>::const_iterator variable  = unknowMap.begin();
                                       variable != unknowMap.end();
                                     ++variable)
  {
    ostringstream * nom = new ostringstream;
    string variableName( variable->second.nom);
    if( variableName.length() > 8 )
    {
      if( verbose==true )
        cerr << "Warning in Model::SurchargeMPSFlux : The variable " << variableName << " has a too long name for the MPS format. This name will be replaced by ";
      variableName = Nom("_",variable->first)();
      if( verbose==true )
        cerr << variableName << "." << endl;
    }
    *nom << variableName;
    tempM[variable->first] = nom;
    unsigned i = (variable->second.variable.GetType() == MuteVar::INT?0:1);
    generals[i].push_back(nom);
  }

   std::list<ExprNode>::const_iterator iter;
   float signe = (isMax?-1.0:1.0);

   for( iter = objectif.nodeList.begin(); iter != objectif.nodeList.end(); ++iter)
   {
      NumVar varTmp = iter->GetVariable();
      VarId varId = (varTmp.GetModel()?varTmp.GetVarId():0);
      
      iterStr = tempM.find(varId);
      if( iterStr != tempM.end() )
      {
         float coef = iter->GetCoefficient().GetValue();
         *(iterStr->second)  << " " << objName << " " << signe*coef;
      }
   }

  VarId p = A.nbLignes+1;
  string strConstraintName;
  for(map< Id,float,Compare<float> >::const_iterator iter  = A.matrix.begin();
                                                     iter != A.matrix.end();
                                                   ++iter)
  {
      unsigned i = iter->first.i; //colonne
      unsigned j = iter->first.j; //ligne

      if(j != p)  //nouvelle contrainte
      {
          constraint = constraintMap.find(j);
          p = constraint->first;

          strConstraintName = constraint->second.name;
          if( strConstraintName.length() > 8 )
          {
            strConstraintName = Nom("_",j)();
          }
      }

      float coeff = iter->second;

      iterStr = tempM.find((VarId)i);
      if( iterStr != tempM.end() )
      {
         *(iterStr->second)  << " " << strConstraintName << " " << coeff;
      }
  }

  flux << "COLUMNS" << endl;
  string markers[2];
  if( generals[0].size() != 0 )
  {
  	markers[0] = "    MARK0000  'MARKER'                 'INTORG'";
  	markers[1] = "    MARK0000  'MARKER'                 'INTEND'";
  }
  else
  {
  	markers[0] = "";
  	markers[1] = "";
  }


  for( unsigned i=0; i < 2; ++ i)
  {
    flux << markers[i] << endl;
    for( iterlistStr = generals[i].begin(); iterlistStr != generals[i].end();  ++iterlistStr )
    {
      istringstream chaine((*iterlistStr)->str());
      string variable;
      string row;
      string value;
      string row2;
      string value2;
  
      chaine >> variable;
      
      variable+=string(8-variable.length(),' ') ;
  
      while( chaine >> row >> value )
      {
        if( !(chaine >> row2 >> value2) )
        {
          row2="";
          value2="";
        }
        row+=string(8-row.length(),' ') ;
        value+=string(14-value.length(),' ') ;
        row2+=string(8-row2.length(),' ') ;
        value2+=string(14-value2.length(),' ') ;
        flux << "    " << variable
             << "  " << row
             << "  " << value
             << "  " << row2
             << "  " << value2
             << endl;
      }
      delete *iterlistStr;
    }
  }

  flux << "RHS" << endl;
  for( constraint = constraintMap.begin(); constraint != constraintMap.end(); ++constraint )
  {
    string strConstraintName(constraint->second.name);
    if( strConstraintName.length() > 8 )
    {
      strConstraintName = Nom("_",constraint->first)();
    }
    strConstraintName+=string(8-strConstraintName.length(),' ');
    flux << "    rhs       " << strConstraintName << "  " << constraint->second.b << endl;
  }

  flux << "BOUNDS" << endl;
  for(map<VarId,Paire>::const_iterator variable  = unknowMap.begin();
                                       variable != unknowMap.end();
                                     ++variable)
  {
   	float lb = variable->second.variable.GetLowerBound();
   	float ub = variable->second.variable.GetUpperBound();
    string variableName( variable->second.nom);
    if( variableName.length() > 8 )
    {
      variableName = Nom("_",variable->first)();
    }
    variableName+=string(8-variableName.length(),' ');

    if( variable->second.variable.GetType() == MuteVar::BOOL )
    {
          flux << " BV bnd       " << variableName  << endl;
    }
    else
    {
    	if( lb != -Infinity || ub != Infinity )
      {
      	if( lb != -Infinity  )
        {
          if( !(lb == 0 && ub == Infinity) )
          {
            flux << " LO bnd       " << variableName << "  " << lb << endl;
          }
        }
        else
        {
          flux << " MI bnd       " << variableName << endl;
        }
        if( ub != Infinity )
        {
          flux << " UP bnd       " << variableName << "  " << ub << endl;
        }
      }
      else
      {
          flux << " FR bnd       " << variableName  << endl;
      }
    }
  }
  flux << "ENDATA" << endl;

  return flux;
}

/**Surcharge de l'operateur de flux
  @param flux le flux  surcharger
  @param m le modele  afficher
  @return Le flux surcharg
 */
std::ostream & ToMPS(std::ostream& flux, const Model& m, bool verbose)
{
  return m.SurchargeMPSFlux(flux,verbose);
}

/*-----------------------------------------------------------------------------------------------*/
/**Permet d'obtenir la contrainte en fonction de son identifiant
  @param id l'identifiant de contrainte
  @return La contrainte correspondant  l'identifiant
 */
Constraint Model::GetConstraint(VarId id) const
{
    map<VarId,InternalConstraint>::const_iterator constraint;
    constraint = constraintMap.find(id);
    //on cherche la contraint
    if(constraint != constraintMap.end())
    {
    	return Constraint(*this,id);
    }
    else
        cerr << "Warning in Model::GetConstraint : undefined id " << id << endl;
    return Constraint();
}

/**Relaxe une contrainte
  @param id l'identifiant de contrainte
  @param lagrangianValue
  @return Vrai si la contrainte a ete relaxee avec succes
  */
bool Model::RelaxConstraint( VarId id, float lagrangianValue )
{
    map<VarId,InternalConstraint>::iterator constraint;
    //on va chercher la contrainte d'identifiant id
    constraint = constraintMap.find(id);
    //si on la trouve
    if(constraint != constraintMap.end())
    {
        isRelaxed = true;
        //on indique au modele qu'il est relaxee
        constraint->second.lagrangianValue = lagrangianValue;
        //on attribue la valeur du lagrangien
        //si elle n'est pas -infini
        if(lagrangianValue != -Infinity)
           constraint->second.isLagrangianParamDefined = true ;
           //le multiplicateur de lagrange est defini
        constraint->second.isLagrangianParamActive = true;
        //le multiplicateur de lagrange devient actif
        objectif-=GetRelaxedConstraintExpr(id);
        //on modifie la fonction objectif
        objectif.isParametric = true;
        //on lui indique qu'elle depend maintenant d'un multiplicateur de lagrange
        objectif.sansConstantes = true;
        //on lui indique de ne pas tenir compte des constantes qu'elle contient
       	return true;
    }
    else
       cerr << "Warning in Model::RelaxConstraint: undefined constraint id " << id << endl;
    return false;
}

/**
  Permet de d'avoir l'evaluation de l'expression linéaire d'une contrainte
  utile dans la resolution lagrangienne
  @param id  l'identifiant de la contrainte
  @return l'expression de la contrainte relaxée
*/
float Model::EvalConstraint(VarId id ) const
{
  float ret = 0.0f;

  unsigned p = id;
  std::map< Id,float,Compare<float> >::const_iterator iter  = A.matrix.begin();
  while(iter != A.matrix.end())
  {
     unsigned i = iter->first.i; //indice de colonne
     unsigned j = iter->first.j; //indice de ligne

     //si on est sur la ligne de la contrainte
     if(j == p)
     {
         ret += iter->second*GetMuteVar(i).GetValue();
         //on ajoute le produit du coefficient et de la valeur
         //de la variable au reel de retour
     }
     else
     if(j>p)
       break;
       //si on depasse la contrainte on quitte

     ++iter;
  }
  return ret;
}

/**
  Permet de d'avoir l'expression relaxe une contrainte
  @param id  l'identifiant de la contrainte
  @return l'expression de la contrainte relaxe
*/
Expr Model::GetRelaxedConstraintExpr( VarId id , bool relaxed)
{

  Expr ret;
  //expression de retour

  LagrangianParam mult(*this, id);

  if(!relaxed)
    mult = LagrangianParam();

  //multiplicateur de lagrange
  std::map< Id,float,Compare<float> >::const_iterator iter  = A.matrix.begin();
  //on parcourt la matrice creuse
  while(iter != A.matrix.end())
  {
     unsigned i = iter->first.i; //indice de colonne
     unsigned j = iter->first.j; //indice de ligne

     //si on est sur la bonne ligne
     if(j == id)
     {
         ret.ASS( iter->second, NumVar(this,i), mult);
         //on ajoute le produit du coefficient et de la variable  l'expression
     }
     else
     if(j>id)
       break;
       //si on depasse la ligne de la contrainte on quitte la boucle
     ++iter;
  }
  float b = 0;
  if( ret.Size() != 0)
     b = constraintMap.find(id)->second.b;
  ret.ASS( - b,NumVar(),mult);
  ret.isParametric = true;
  return ret;
}

/**
  Relaxe une contrainte en la multipliant par un multiplicateur de Lagrange et en
  l'additionnant  la fonction objectif
  @param nom  le nom de la contrainte
  @param mult le multiplicateur de Lagrange
  @return vrai si l'operation s'est bien droul
 */
bool Model::Relax(const std::string & name, float lagrangianValue )
{
   VarId id;
   //si on arrive a recuperer l'identifiant de la contrainte
   if((id=GetConstraintId(name)))
     return RelaxConstraint(id,lagrangianValue);
     //on la relaxe
   else
     return false;
}


/*-----------------------------------------------------------------------------------------------*/
/**
  Relaxe les contraintes sur les types de variables ici les variables entieres
  si la variable en parametre est la variable nulle on relaxe toutes les varaibles
  autrement dit RelaxInt2Float() relaxe toutes les variables
  @param la variable a relaxee
  @return vrai si l'operation s'est bien droule
 */
bool Model::RelaxInt2Float( NumVar numVar )
{
  VarId id = numVar.GetVarId();
  //on recupere l'identifiant de la variable

  //si l'identifiant n'est pas nul
  if(id)
     unknowMap.find(id)->second.variable.SetRelaxed();
     //on relaxe la variable correspondante
  else
    //sinon pour chaque variables
    for(map<VarId,Paire>::iterator variable  = unknowMap.begin();
                                        variable != unknowMap.end();
                                      ++variable)
    {
    	//on regarde si elle est de type entier
        if(variable->second.variable.GetType()==MuteVar::INT)
    	{
    		variable->second.variable.SetRelaxed();
    		//si oui on la relaxe
    	}
    }
  return true;
}

/**
  Relaxe les contraintes sur les types de variables ici les variables boolennes
  si la variable en parametre est la variable nulle on relaxe toutes les varaibles
  autrement dit RelaxBool2Float() relaxe toutes les variables
  @param la variable a relaxee
  @return vrai si l'operation s'est bien droule
 */
bool Model::RelaxBool2Float( NumVar numVar )
{
  VarId id = numVar.GetVarId();
  //On recupere l'identifiant de la variable

  //si l'identifiant n'est pas nul
  if(id)
     unknowMap.find(id)->second.variable.SetRelaxed();
     //on relaxe la varaiable
  else
    //sinon pour chaque variable
    for(map<VarId,Paire>::iterator variable  = unknowMap.begin();
                                        variable != unknowMap.end();
                                      ++variable)
    {
    	//si elle est du type booleen
        if(variable->second.variable.GetType()==MuteVar::BOOL)
    	{
    		variable->second.variable.SetRelaxed();
    		//on la relaxe
    	}
    }
  return true;
}


/**
  Nettoie le problme relax pour qu'il retrouve son aspect d'origine
 */
void Model::UnrelaxModel()
{
   objectif.Simplify(false);
   objectif.isParametric = false;
   objectif.sansConstantes = false;
   //on reinitialise la fonction objectif

   //Pour toutes les variables
   for(map<VarId,Paire>::iterator variable  = unknowMap.begin();
                                  variable != unknowMap.end();
                                ++variable)
    	variable->second.variable.SetRelaxed(false);
    	//on leur enleve le caractere relachee

   //pour toutes les contraintes
   for(map<VarId,InternalConstraint>::iterator constraint  = constraintMap.begin();
                                               constraint != constraintMap.end();
                                             ++constraint)
   {
   	  constraint->second.isLagrangianParamActive = false;
   	  //on desactive leur multiplicateur de lagrange
   }
   isRelaxed=false;
}


//-------------------------
// Interface des Solvers
//-------------------------

/**Definit le solver*/
void  Model::SetAPI(API _API)
{
	solverAPI.SetAPI(_API);
}

/**Resoud le Modele*/
float Model::Solve()
{
	float val;

    ofstream out(solverAPI.GetfLP());
	if(!out.is_open())
	   throw runtime_error("Model::Solve() : I/O Error");
    //on declare un fichier .lp temporaire de sortie
    out << (*this);
    //on met le modele dans ce fichier sous la bonne forme
	out.close();
	//on ferme ce fichier pour eviter des problemes de lecture

    solverAPI.SetMIP( MIPVar == 0);

	val = solverAPI.Solve();
	isSolved = solverAPI.IsSolved();
	isOptimal = solverAPI.IsOptimal();
	//On appelle le solveur sur le fichier LP

	// appel de SetNumVarValues
	// pour stocker les rsultat des varialbes
	SetNumVarValues(solverAPI.GetNumVarNoms(), solverAPI.OutNumVarValues());
	// appel de SetConstraintValues
	// pour stocker les rsultat des contraintes
    SetConstraintValues( solverAPI.GetConstraintNoms(), solverAPI.OutConstraintValues() );
    //on sauve la valeur de la fonction objecitf
    objectifActivity = val + objectif.GetConstant();
    return objectifActivity;
}

/**Remplie les listes listViolated et listSlacked par les coefficients de lagrange correspondant aux contraintes
   respectivement violées et laches
   @param listViolated  liste des contraintes violees
   @param listSlacked liste des contraintes laches
 */
void Model::GetInfluentLagrangianParam( list<LagrangianParam> & listViolated, list<LagrangianParam> & listSlacked)
{
     map<VarId,InternalConstraint>::iterator constraint = constraintMap.begin();
     //Pour toutes les contraintes
     for(;constraint != constraintMap.end(); ++constraint)
     {
          float ecart = constraint->second.b - constraint->second.activity;
          //on calcule la variable d'cart
          if( ecart < 0 )
             listViolated.push_back( LagrangianParam(*this,constraint->first) );
             //la contrainte est viole
          if( ecart > 0 )
             listSlacked.push_back( LagrangianParam(*this,constraint->first) );
             //la contrainte est lache
     }
}

/**Renvoie un vecteur contenant tous les multiplicateurs de lagrange du modele
  @return Un vecteur de taille egal au nombre de contrainte du probleme
  */
std::vector<LagrangianParam> Model::GetAllLagrangianParams( )
{
    vector<LagrangianParam> retour;
    //le vecteur de retour
    map<VarId,InternalConstraint>::iterator constraint = constraintMap.begin();
    //on parcourt le map des contraintes
    for(;constraint != constraintMap.end(); ++constraint)
    {
        retour.push_back( LagrangianParam(*this,constraint->first) );
        //on ajoute le lagrangien de chaque contrainte au vecteur
    }
    return retour; 
}

/**Dfinit les valeurs des contraintes en fonctions de deux listes
   correspondant aux nouveaux couples valeur-nom
   @param noms la liste des noms de variables  changer
   @param values la listes de leurs valeurs reciproques
  */
void Model::SetNumVarValues(const list<string> & noms, const list<float> & values)
{
   map<string,float> resultats;
   //ce map permet de gagner du temps dans la recherche sur la liste de noms
   //il permet de passer d'une complexite en O(n*n)  O(n*Log(n))
   //il fait correspondre une valeur  un nom de contrainte

   //si le nombre de noms correspond au nombre de valeurs
   if(noms.size() == values.size())
   {
       //on construit le map des rsultats d'execution
       list<string>::const_iterator iterNoms = noms.begin();
       //iterateur sur la liste de noms
       list<float>::const_iterator iterValues = values.begin();
       //iterateur sur la liste de valeurs

       //on parcout les deux listes en meme temps
       //sachant qu'on est sur qu'elle font la mme taille
       while( iterNoms != noms.end() )
       {
          resultats[*iterNoms] = *iterValues;
          //on cree le couple ( nom, valeur )
          ++iterNoms;
          ++iterValues;
          //on avance dans les deux listes
       }

       map<string,float>::const_iterator iter;
       //on parcourt le map de variables
       for(map<VarId,Paire>::iterator variable  = unknowMap.begin();
                                      variable != unknowMap.end();
                                    ++variable)
       {
       	 iter = resultats.find(variable->second.nom);
       	 //on cherche si le nom de la variable est dans le map de resultat
         if( iter != resultats.end() )
       	    variable->second.variable = iter->second;
       	    //si oui on attribue la valeur
       	 else
       	    variable->second.variable = 0;
       	    //sinon on la met  nul
       }
   }
}

/**Dfinit les valeurs des contraintes en fonctions de deux listes
   correspondant aux nouveaux couples valeur-nom
   @param noms la liste des noms de variables  changer
   @param values la listes de leurs valeurs reciproques
  */
void Model::SetConstraintValues(const std::list<std::string> & noms, const std::list<float> & values)
{
   map<string,float> resultats;
   //ce map permet de gagner du temps dans la recherche sur la liste de noms
   //il permet de passer d'une complexite en O(n*n)  O(n*Log(n))
   //il fait correspondre une valeur  un nom de contrainte

   //si le nombre de noms correspond au nombre de valeurs
   if(noms.size() == values.size())
   {
       //on construit le map des rsultats d'execution
       list<string>::const_iterator iterNoms = noms.begin();
       //iterateur sur la liste de noms
       list<float>::const_iterator iterValues = values.begin();
       //iterateur sur la liste de valeurs

       //on parcout les deux listes en meme temps
       //sachant qu'on est sur qu'elle font la mme taille
       while( iterNoms != noms.end() )
       {
          resultats[*iterNoms] = *iterValues;
          //on cree le couple ( nom, valeur )
          ++iterNoms;
          ++iterValues;
          //on avance dans les deux listes
       }

       map<string,float>::iterator trouve;
       map<VarId,InternalConstraint>::iterator constraint = constraintMap.begin();
       //on parcourt le map des contraintes
       for(;constraint != constraintMap.end(); ++constraint)
       {
            trouve = resultats.find(constraint->second.name);
            //si on trouve le nom de la contrainte dans le map de resultats
            if( trouve != resultats.end())
                constraint->second.activity = trouve->second;
                //on lui attribue sa valeur
            else
                constraint->second.activity = EvalConstraint(constraint->first);
                //sinon on la calcule
       }
   }
   else
   {
   	  cerr << "Warning in Model::SetConstraintValues : list dimension problem, ";
   	  if( noms.size() <= values.size() )
    	  cerr << "the first param is too large" << endl;
      else
    	  cerr << "the second param is too large" << endl;
   }

}

/**Sortie sur cout des valeurs des variables
  @param withoutNullValue qui spcifie si les variables nulles doivent etre affichees
  */
std::ostream & Model::OutNumVarValues( bool withoutNullValue,std::ostream & flux, const std::string & newline) const
{
    float t=0.0;
    string strInf = "Infinity";
    if( newline == "<br/>")
      strInf = "<em>oo</em>";
    else if( newline == "\\\\")
           strInf = "$\\infty$";


    flux << "Valeurs des variables (type et domaine de dfinition):" << newline << endl;
    //on parcourt le map des variables et on affiche celles qui sont non nulles
    //s'il y a l'option withoutNullValue
    for(map<VarId,Paire>::const_iterator variable  = unknowMap.begin();
                                          variable != unknowMap.end();
                                        ++variable)
    {
        if ( (t = (variable->second.variable.GetValue())) || !withoutNullValue)
        {
            int n=1,s=1;
            float e=0;
            float residu=0.0;
            unsigned d=1;
            bool aviableResidu = false;

            if (isFract==true)
            {
               s = (t<0.0?-1:1);
               e = floor(t);
               residu = t - e;
               aviableResidu = _fract.Approx(residu,n,d);
            }
            flux << variable->second.nom << " = ";
             if (isFract==true && aviableResidu==true)
             {
               if (s<0)
                  flux << '-' << (e!=0?e:' ') << '-' << n<<'/'<<d;
               else
                  flux << (e!=0?e:' ') << '+' << n << '/' << d;
             }
             else
               flux << t;
            // Affichage des bornes
            float uB = variable->second.variable.GetUpperBound();
            float lB = variable->second.variable.GetLowerBound();
            ostringstream sstr_lB; sstr_lB << lB;
            ostringstream sstr_uB; sstr_uB << uB;
            string type;

            switch(variable->second.variable.GetType())
            {
              case MuteVar::INT:   type = "INT";   break;
              case MuteVar::FLOAT: type = "FLOAT"; break;
              case MuteVar::BOOL:  type = "BOOL";  break;
              default:
                	cerr << "Warning in Model::OutNumVarValues : Error type of variable in switch statement " << endl;
             	break;
            }

            flux << "  ........ "<< type <<" ....... [" << (lB==-Infinity?"-"+strInf : sstr_lB.str()) << ',' << (uB==Infinity ? "+"+strInf : sstr_uB.str()) << ']';
            flux << newline << endl;
        }
    }
    return flux;
}

/**Affiche les valeurs des contraintes et leurs tats*/
std::ostream & Model::OutConstraintValues(std::ostream & flux, const std::string & newline ) const
{
     flux << "Caracteristiques des constraintes (si non toutes satures): " << newline << endl;
     map<VarId,InternalConstraint>::const_iterator constraint = constraintMap.begin();
     //on parcourt le map de contraintes
     string op[]  = { "<=",">=","=",">","<"};

     for(;constraint != constraintMap.end(); ++constraint)
     {
            //on affiche la valeur de la contrainte apres execution

            float ecart = constraint->second.b - constraint->second.activity;
            if(  constraint->second.type == InternalConstraint::SUP
              || constraint->second.type == InternalConstraint::SUPSTRICT )
            	ecart = -ecart;


            // Conversion en fraction
            int n=1,s=1;
            float e=0;
            float residu=0.0;
            unsigned d=1;
            bool aviableResidu = false;

            if (isFract==true) 
            {
               s = (ecart<0.0?-1:1);
               e = floor(ecart);
               residu = (s<0 ? (-ecart)-e : ecart-e);
               aviableResidu = _fract.Approx(residu,n,d);
            }
            //on regarde la variable d'cart et on tudie les trois cas qui se prsentent
            if ( -1e-10 >= ecart || ecart >= 1e-10)
            {
                flux << constraint->second.name << " = " << constraint->second.activity;

                if( ecart > 0 ) {
                   if (isFract==true && aviableResidu==true)
                   {
                     if (s<0)
                        flux << (e!=0.0?'-'<<int(e):' ') << '-' << n<<'/'<<d;
                     else
                        flux << (e!=0.0?e:' ') << '+' << n << '/' << d;
                   }
                   else
                     flux << " " <<op[constraint->second.type] << " "
                     << constraint->second.b << " ... non saturee ecart : " << ecart
                     << newline << endl;
                }
                else {
                   if (isFract==true && aviableResidu==true)
                   {
                     flux << "contrainte ecart: ";
                     if (s<0)
                        flux << (e!=0.0?'-'<<int(e):' ') << '-' << n<<'/'<<d;
                     else
                        flux << (e!=0.0?e:' ') << '+' << n << '/' << d;
                   }
                   else
                     flux << " " <<op[constraint->second.type] << " "
                     << constraint->second.b << " ... violee ecart : " << ecart
                     << newline << endl;
                }
                
            }
     }
     return flux;
}

/**Chargement d'un fichier LP  partir duquel on va charger le modele
   @param _name le nom du fichier LP de destination
   @param verbose un flag d'affichage
   @return Vrai si le chargement c'est bien pass
 */
bool Model::LoadLP(const std::string& _name, bool verbose)
{
    FichierLP file;
    //on dclare un fichier LP source

	//si on ne reussi pas  charger le fichier LP
    if(!file.Open(_name.c_str()))
    {
    	cerr << "Warning in Model::LoadLP : I/O error in the opening of " << _name << endl;
        return false;
        //on quitte
	}
	
    //sinon on charge le modle  partir du fichier LP
    if( file >> (*this) )
    {
        //si le flag d'affichage est vrai
        if(verbose == true)
        {
            cout << file << endl;
            //on affiche
        }
        return true;
    }
    return false;
}

/**Chargement d'un fichier MPS  partir duquel on va charger le modele
   @param _name le nom du fichier MPS de destination
   @param verbose un flag d'affichage
   @return Vrai si le chargement c'est bien pass
 */
bool Model::LoadMPS(const std::string& _name, bool verbose)
{
    FichierMPS file;
    //on dclare un fichier LP source

	//si on ne reussi pas  charger le fichier LP
    if(!file.Open(_name.c_str()))
    {
    	cerr << "Warning in Model::LoadLP : I/O error in the opening of " << _name << endl;
        return false;
        //on quitte
	}
	
    //sinon on charge le modle  partir du fichier LP
    if( file >> (*this) )
    {
        //si le flag d'affichage est vrai
        if(verbose == true)
        {
            cout << file << endl;
            //on affiche
        }
        return true;
    }
    return false;
}

/**Affiche les principales caractristiques du problme
   @param withoutNullValue permet de ne pas afficher les variables de valeurs nulles
 */
void Model::ShowMe( bool withoutNullValue)
{
     OutNumVarValues(withoutNullValue);
     OutConstraintValues();
}

/**Enlve une variable du modle
 @param varId l'identifiant de la variable dans le modle
 */
void Model::RemoveVariable(VarId varId)
{
   map<VarId,Paire>::iterator variable = unknowMap.find(varId);
   //on regarde si l'identifiant existe
   if( variable != unknowMap.end() )
   {
   	 unknowMap.erase( variable );
   	 //on efface la variable associe puis on nettoie la matrice creuse

     std::map< Id,float,Compare<float> >::iterator iter  = A.matrix.begin();
     std::map< Id,float,Compare<float> >::iterator tmp;

     //on parcourt la matrice
     while(iter != A.matrix.end())
     {
       unsigned i = iter->first.i; //indice de colonne

       //si l'on se trouve sur la colonne de la matrice creuse
       if(i == varId)
       {
        //dans de ca on supprime l'element
        tmp = iter;
        //mais pas directement car erase ne retourne la position du prochain lment
        ++iter;
        //donc on avance
        A.matrix.erase( tmp );
        //et ensuite on supprime
       }
       else
          ++iter;
          //sinon on se contente d'avancer
     }
   }
   else
     cerr << "Warning in Model::RemoveVariable : undefined id " << varId << endl;

}

/**Enlve une contrainte du modle
  @param id la contrainte du modle
 */
void Model::RemoveConstraint(VarId id)
{

  std::map< Id,float,Compare<float> >::iterator iter  = A.matrix.begin();
  //iterateur principal qui va parcourir le map matrix de A
  std::map< Id,float,Compare<float> >::iterator tmp;
  //iterateur secondaire qui sert  sauvegarder les positions  supprimer
  //tant que l'on a pas fini de percourir le map
  while(iter != A.matrix.end())
  {
     unsigned j = iter->first.j;
     //on recupere l'indice horizontale de l'lment courant
     //on le compare avec l'identifiant de la contrainte
     if(j == id)
     {
        //dans de ca on supprime l'element
        tmp = iter;
        //mais pas directement car erase ne retourne la position du prochain lment
        ++iter;
        //donc on avance
        A.matrix.erase( tmp );
        //et ensuite on supprime
     }
     else
     {
       //sinon on n'est pas sur la bonne contrainte
       ++iter;
       //on avance
       //sauf si on a pass la contrainte
       if(j>id)
         //dans ce cas on quitte la boucle
         break;
     }
  }
}

/**Ajoute une expression linaire  la parite linaire de la contrainte
  @param id l'identifiant de la contrainte
  @param expr l'expression linaire  rajouter
 */
void Model::AddToConstraint(VarId id ,const Expr & expr)
{
   std::list<ExprNode>::const_iterator iter;
   //itrateur constant sur l'expression expr constante
   //si la contrainte existe
   if( constraintMap.find(id) != constraintMap.end() )
   {
     //on parcourt l'expression
     for( iter = expr.nodeList.begin(); iter != expr.nodeList.end(); ++iter)
     {
        float coef = iter->GetCoefficient().GetValue();
        NumVar varTmp = iter->GetVariable();
        VarId varId = (varTmp.GetModel()?varTmp.GetVarId():0);
        //on rcupere l'identifiant de la variable
        //si il est non nul et que la variable est dfinie
        if(varId && unknowMap.find(varId) != unknowMap.end() )
        {
          float acoef = A.Get(varId,id);
          //on rcupre le coeffictient dans la matrice
          //si il est diffrent de l'oppos du nouveau coefficient
          if(acoef != -coef)
            A.Set(varId,id,coef+acoef);
            //coef <- coef + nouveau coef
          else
            A.matrix.erase(A.matrix.find(Id(varId,id)));
            //sinon le coef devient nul et donc on le supprime
        }
        else
        {
           if( varId )
              cerr << "Warning in Model::AddToConstraint : undefined varId " << varId << endl;
        }
      }
   }
   else
     cerr << "Warning in Model::AddToConstraint : undefined constraint id " << id << endl;
}

/**Rcupre une contrainte  partir de son nom
  @param name le nom de la contrainte
  @return La contrainte correspondante au nom
  */
Constraint Model::GetConstraint(const string & name)
{
  //on cherche son identifiant puis on renvoie la contrainte
  return GetConstraint(GetConstraintId(name));
}

/**Renvoie l'identifiant de la contrainte en fonction du nom
   Cette recherche est lente car le nom peut etre n'importe ou
   dans le stockage optimiser pour les recherches suivant l'identifiant.
   En cas d'echec on retourne 0
  @param name  le nom de la contrainte
  @return L'identifiant de la contrainte nomme name
  */
VarId Model::GetConstraintId(const string & name)
{
    bool trouve = false; //boolen servant de "break"  la boucle de recherche

    VarId id = 0; //l'identifiant est nul si on ne trouve pas de contrainte

    map<VarId,InternalConstraint>::iterator constraint = constraintMap.begin();
    //iterateur de map initialis au dbut
    //tant que l'on a pas trouv ou qu'il reste des contraintes  tester
   	while(!trouve && constraint != constraintMap.end() )
   	{
        //on verifie si l'on a trouv la contrainte suivante
        if( ( constraint->second.name == name) )
        {
          id = constraint->first;
          //on rcupre l'identifiant de la contrainte qui est la cl dans le constraintMap
          trouve = true;
          //on quitte la boucle a la fin de cette itration
        }
        ++constraint;
        //on passe  la contrainte suivante
    }
    return id;
}

/**Renomme une contrainte
  @param id l'identifiant de la contrainte  renommer
  @param name son nouveau nom
 */
void Model::RenameConstraint( VarId id, const string & name)
{
    map<VarId,InternalConstraint>::iterator constraint = constraintMap.find(id);
    //on regarde si la contrainte existe
    if( constraint != constraintMap.end())
        //si oui on reattribue le nom
        constraint->second.name = name;
    else
        cerr << "Warning in Model::RenameConstraint : undefined id " << id << endl;
}

/**Change le coefficient de la contrainte d'identifiant VarId
  @param id l'identifiant de la contrainte
  @param bid le nouveau coefficient de la contrainte
 */
void Model::SetSecondMember(VarId id ,float bid)
{
    map<VarId,InternalConstraint>::iterator constraint = constraintMap.find(id);
    //on regarde si la contrainte existe
    if( constraint != constraintMap.end())
        //si oui on change le coefficient
        constraint->second.b = bid;
    else
        cerr << "Warning in Model::RenameConstraint : undefined id " << id << endl;
}

/**retourne le coefficient de la contrainte d'identifiant VarId
  @param id l'identifiant de la contrainte
  @return Le coefficient de la contrainte
 */
float Model::GetSecondMember(VarId id) const
{    
    map<VarId,InternalConstraint>::const_iterator constraint = constraintMap.find(id);
    //on regarde si la contrainte existe
    if( constraint != constraintMap.end())
        //si oui on change le coefficient
        return constraint->second.b;
    else
        cerr << "Warning in Model::RenameConstraint : undefined id " << id << endl;
    return Infinity;
}

/**Nettoye le modele*/
void Model::Clear()
{
    hacheVarId=1;
    hacheConstraintId=1;
    unknowMap.clear();
    A.matrix.clear();
    A.nbLignes = A.nbCols = 0;
    constraintMap.clear();
    comments.clear();

    isMax = true;
    isRelaxed = false;
    isFract = false;
    isOptimal = false;
    isSolved = false;
    isNormalized = false;
    MIPVar = 0;

    objectif = Expr();
    objectifActivity = 0.0f;
    objectifName = "";
    problemName = "";
}

void Model::Normalize(bool withoutStrict)
{
    unsigned p = A.GetNbLignes()+1;
    float coef = 1.0f;
    
    if(isMax==true)
    {
    	objectif*=-1;
    	isMax=false;
    }


    map<VarId,InternalConstraint>::iterator constraint;
    for(map< Id,float,Compare<float> >::iterator iter  = A.matrix.begin();
                                                 iter != A.matrix.end();
                                               ++iter)
    {
        unsigned i = iter->first.i; //indice de colonne
        unsigned j = iter->first.j; //indice de ligne
        if( p != j)
        {
        	constraint = constraintMap.find(j);
            switch( constraint->second.type )
            {
                case InternalConstraint::SUP :
                    coef = 1.0f;
                break;

                case InternalConstraint::SUPSTRICT :
                    coef = 1.0f;
                    constraint->second.type = InternalConstraint::SUP;
                break;

                case InternalConstraint::INF :
                    coef = -1.0f;
                    constraint->second.type = InternalConstraint::SUP;
                break;

                case InternalConstraint::INFSTRICT :
                    coef = -1.0f;
                    constraint->second.type = InternalConstraint::SUP;
                break;
                case InternalConstraint::EGAL :
                    coef = 1.0;
                break;
                default:
                break;
             }
             constraint->second.b*=coef;
        	 p = j;
        }
        map<VarId,Paire>::iterator variable =  unknowMap.find(i);
        if( variable->second.variable.GetLowerBound() >=0 )
            iter->second*=coef;
        else
            iter->second*=-coef;
    }
     map<VarId,Paire>::iterator variable;
     for(variable = unknowMap.begin();variable != unknowMap.end();++variable)
     {
        if( !(variable->second.variable.GetLowerBound() >=0 ))
        {
           float lb = variable->second.variable.GetLowerBound();
           float ub = variable->second.variable.GetUpperBound();
           variable->second.variable.SetLowerBound(-ub);
           variable->second.variable.SetUpperBound(-lb);
        }
     }

}




/**Permet de calculer le dual du programme dans un Model
  @return Le Model correspondant au programme linaire
  */
Model Model::GetDual() const
{
    Model ret;
    map<VarId,Paire>::const_iterator variable;
    map<VarId,InternalConstraint>::const_iterator constraint;

    ret.isMax = !isMax;
    ret.solverAPI = solverAPI;
    ret.objectifName = objectifName + "dual";
    ret.problemName = problemName + "dual";

    for(map< Id,float,Compare<float> >::const_iterator iter  = A.matrix.begin();
                                                         iter != A.matrix.end();
                                                       ++iter)
    {

        unsigned i = iter->first.i; //indice de colonne
        unsigned j = iter->first.j; //indice de ligne
        float coef = iter->second;
        ret.A.Set(j,i,coef);
    }
    
    for(constraint = constraintMap.begin(); constraint != constraintMap.end(); ++constraint)
    {
       unsigned id = constraint->first;
       Paire var;
       InternalConstraint ic = constraint->second;
       var.nom = ic.name;
       float lb=0,ub=Infinity;
       var.variable = MuteVar( MuteVar::FLOAT,ic.activity,lb,ub);
       ret.unknowMap[id] = var;
       ret.objectif += ic.b * NumVar( const_cast<const Model *>(&ret), id);
     }

    for(variable = unknowMap.begin(); variable != unknowMap.end(); ++variable)
    {
       unsigned id = variable->first;
       ret.constraintMap[id] = InternalConstraint( InternalConstraint::INF,
                                                   0.0,
                                                   variable->second.variable.GetValue(),
                                                   0.0,
                                                   variable->second.nom);

    }

    for( list<ExprNode>::const_iterator iter = objectif.nodeList.begin();
                                        iter != objectif.nodeList.end();
                                      ++iter)
    {
    	float b = iter->GetCoefficient().GetValue();
    	unsigned id = iter->GetVariable().GetVarId();
        ret.SetSecondMember(id,b);
    }

    return ret;
}

/**Permet de rcuperer le dual du programme linaire dans un fichier
  @param fileName le fichier de sortie
  */
void Model::PrintDual(const std::string& fileName) const
{
    ofstream file( fileName.c_str() );
    if(!file.is_open())
        throw runtime_error("Model::PrintDual : I/O Error");
    file << GetDual();
    file.close();
}


/**Ajout d'un commentaire avant la prochaine contrainte
   si un commentaire tait dj la, il sera cras, mais aussi
   retourn
   @param comment  la chaine de commentaire sans le "\"
   @retrurn l'ancien commentaire
 */
std::string Model::AddComment( const std::string & comment )
{
  std::map<VarId,std::string>::iterator trouve = comments.find(hacheConstraintId);
  std::string retour;
  if(trouve!=comments.end())
  {
  	retour = trouve->second;
    trouve->second = comment;
  }
  else
    comments[hacheConstraintId] = comment;
  return retour;
}

/**Dfinit un commentaire pour la contrainte d'identifiant id
   si un commentaire tait dj la, il sera cras
   @param id l'identifiant de la contrainte
   @param comment  la chaine de commentaire sans le "\"
 */
void Model::SetComment( VarId id, const std::string &  comment)
{
  std::map<VarId,std::string>::iterator trouve = comments.find(id);
  if(trouve!=comments.end())
     trouve->second = comment;
  else
     comments[id] = comment;
}

std::string Model::GetComment( VarId id ) const
{
  std::map<VarId,std::string>::const_iterator trouve = comments.find(id);
  if(trouve!=comments.end())
     return trouve->second;
  else
     return std::string();
}


#ifdef USE_HTML_FORMAT
/**Cration d'un document au format html
  @param fileName le nom du document
*/
void Model::SortieHTML( const std::string & fileName, const std::string& matrixPNG)
{
    HTMLFile html(fileName);
    std::ostream & out = html.getStream();
    string s1,s2;
    SolveInfos(s1,s2);

    // Si ce n'est pas rsolu, on l'affiche en gros et en haut
    if (isSolved == false)
    {
        html << "<h3>Probl&egrave;me non r&eacute;solu</h3>";
        html.NewLine();
    }

    // Entete html/liens
    html << "<h1>Sortie du Mod&egrave;le du Programme lin&eacute;aire</h1>";
    html << "<ul>";
    html << "<li><a href=\"#caracteristique_probleme\">Caract&eacute;ristiques du probl&egrave;me</a></li>";
    html << "<li><a href=\"#dimension_probleme\">Dimensions du probl&egrave;me</a></li>";
    if( A.GetNbLignes() < 20 && A.GetNbCols() < 20)
      html << "<li><a href=\"#pl\">Programme linaire</a></li>";
    if( isSolved )
      html << "<li><a href=\"#resolution\">R&eacute;solution du probl&egrave;me</a></li>";
    if (matrixPNG != "")
      html << "<li><a href=\"#matrice_creuse\">Affichage de la matrice creuse</a></li>";

    html << "</ul>";
    html.NewLine();
    html << "<h2><a name=\"caracteristique_probleme\">Caract&eacute;ristiques du probl&egrave;me</a></h2>";

    html << "Probl&egrave;me de <b>" << (isMax?"Maximization":"Minimization") << "</b>";
    html.NewLine();
    if( problemName != "" )
    {
        html << "Nom du probl&egrave;me: \"" << problemName << "\"";
        html.NewLine();
    }
    if( objectifName != "" )
    {
      html << "Nom de la fonction objectif: <b>" << objectifName << "</b>";
      html.NewLine();
    }
    html << "<br />";
    html << "<h2><a name=\"dimension_probleme\">Dimensions du probl&egrave;me</a></h2>";
    html.NewLine();
    OutDimensions(out,"<br/>");
    html << "<br />";
    // Sortie du LP dans le fichier HTML
    if( A.GetNbLignes() < 20 && A.GetNbCols() < 20)
    {
        html << "<h2><a name=\"pl\">Programme lin&eacute;aire</a></h2>";
        html << *this;
        html << "<br />";
    }
    if( isSolved )
    {
        html << "<h2><a name=\"resolution\">R&eacute;solution du probl&egrave;me</a></h2>";
        html.NewLine();
        html << "Le probl&egrave;me a &eacute;t&eacute; r&eacute;solu avec " << (solverAPI.GetAPI()==1?"GLPSOL" : "CPLEX");
        html.NewLine();
        html.NewLine();
        if (isRelaxed) {
           html << "Le probl&egrave;me est relax&eacute;";
           html.NewLine();
        }
        if( isOptimal )
            html << "Solution <b>OPTIMALE</b>: " << objectifActivity;
        else
            html << "Solution NON OPTIMALE: " << objectifActivity;
        html.NewLine();
        html << "Sortie solveur: " << s2;

        html.NewLine();
        html.NewLine();
        OutNumVarValues(true,out,"<br/>");
        html.NewLine();
        OutConstraintValues(out,"<br/>");
        html.NewLine();

        // Si on a sorti un matrice creuse, on peut la mettre dans le fichier HTML
        #ifdef EXPORT_SPARSE_MATRIX_PNG
        if (matrixPNG != "")
        {
            html << "<h2><a name=\"matrice_creuse\">Affichage de la matrice creuse</a></h2>";
            html << "<div align='center'>";
            html << "<img src='./" << matrixPNG << "' alt\"MatriceCreuse\"/>";
            html << "</div>";
            html.NewLine();
        }
        #endif
    }
    html.Close();
}
#endif

#ifdef USE_LATEX_FORMAT
/**Cration d'un document au format LaTeX
  @param fileName le nom du document
*/
void Model::SortieLATEX( const std::string & fileName)
{
    LATEXFile latex(fileName,"Programme linaire",false);
    std::ostream & out = latex.getStream();
    latex << "Sortie du Modele du Programme linaire";
    latex.MakeSection("Caractristiques du problme");

    if( problemName != "" )
    {
        latex.MakeSection("Nom du problme:");
        latex << "  Le nom du probleme est" << problemName;
        latex.EndSection();
    }
    latex.MakeSection( "Dimensions du problme:");
    OutDimensions(out,"\\\\");
    latex.EndSection();
    latex.EndSection();
    if( A.GetNbLignes() < 20 && A.GetNbCols() < 20)
    {
        latex.MakeSection( "Programme linaire");
        latex << *this;
        latex.EndSection();
    }
    if( isSolved )
    {
        latex.MakeSection( "Rsolution du problme");
        latex.MakeSection( "Fonction Objectif");
        if( isOptimal )
            latex << "Solution OPTIMALE: " << objectifActivity;
        else
            latex << "Solution NON OPTIMALE: " << objectifActivity;
        latex.EndSection();

        latex.MakeSection( "Variables");
        OutNumVarValues(true,out,"\\\\");
        latex.EndSection();
        latex.MakeSection( "Contraintes");
        OutConstraintValues(out,"\\\\");
        latex.EndSection();
    }
    else
    {
        latex << "Problme non rsolu";
    }
    latex.Close();
}
#endif

#ifdef EXPORT_SPARSE_MATRIX_PNG

  #include "Png.h"

  void Model::ExportSparseMatrixPNG(const std::string& _nom) const
  {
    // Obtenir la dimension de la matrice
    unsigned w=A.GetNbCols(),h=A.GetNbLignes();
    // dfinir une quantit de Zoom sur l'image p-
    unsigned zoom=1;

    PNGImage image(_nom.c_str(), w, h);
    image.openFileWrite();
    image.initWriteStructs();
    image.writeHeader();
    png_byte **Buff = image.writeImage_Start();
    unsigned char col1=255,col2=255,col3=255;

    for (unsigned y=0;y<h;y+=zoom)
        for (unsigned x=0;x<w;x+=zoom)
        {
          float val = A(x,y);
      		col2 = 65;
      		//cout << scaled << " <-> " << val << " | ";
      		if (val < -epsylon)    {col1 = 255; col3=0;} // rouge = ngatif
      		else if(val > epsylon) {col1=0; col3 = 255;} // bleu  = positif
      		else
      		{
      		  col1=col2=col3=255;
      		}
          Buff[y][x*3+0] = col1;
          Buff[y][x*3+1] = col2;
          Buff[y][x*3+2] = col3;
        }

    image.writeImage_End();
    image.writeEnd();
  }

#endif

#ifdef COMPILE_FOR_PYTHON
	#include "py_Model.cxx"
#endif


}
