/*
	Copyright (c) 2005, Quentin Lequy and Romain Gaucher
	All rights reserved.

	Redistribution and use in source and binary forms, with or 
	without modification, are permitted provided that the following 
	conditions are met:

		* Redistributions of source code must retain the above copyright 
		  notice, this list of conditions and the following disclaimer.

		* Redistributions in binary form must reproduce the above copyright 
		  notice, this list of conditions and the following disclaimer in the 
		  documentation and/or other materials provided with the distribution.

		* Neither the name of Quentin Lequy or Romain Gaucher nor the names 
		  of its contributors may be used to endorse or promote products 
		  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
	AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "NumVarMatrix.h"
#include "NumVarArray.h"
#include "Model.h"

namespace Modelib {

/*----------------------------------------------------------------------------*/

/**
  Constructeur principal\n
  Sert principalement  gnrer les noms des variables NumVar en fonction du nom de dpart
  de la famille.
*/

NumVarMatrix::NumVarMatrix( Model & _modele,
                            unsigned sizeI,
                            unsigned sizeJ,
                            float _lb,
                            float _ub,
                            MuteVar::VarType _type,
                            const std::string & nom,
                            const std::string & milieu,
                            const std::string & fin)
: modele(&_modele),type(_type)
{

  if(nom != "unknow")
    for(unsigned i=0;i < sizeI; ++i)
    {
      char id[16];
      sprintf(id,"%d",i);
      varMatrix.push_back(new NumVarArray(_modele, sizeJ, _lb, _ub, _type, nom+id+milieu, fin));
    }
  else
  for(unsigned i=0;i < sizeI; ++i)
    varMatrix.push_back(new NumVarArray(_modele, sizeJ, _lb, _ub, _type));
}

/**
  Constructeur de recopie
*/
NumVarMatrix::NumVarMatrix( const NumVarMatrix & nva )
:type(nva.type)
{
  for(unsigned i = 0; i < nva.varMatrix.size(); ++i)
    varMatrix.push_back(new NumVarArray(*(nva.varMatrix[i])));
}

/**
    Constructeur crant un NumVarArray a partir d'un vecteur de variable
    @param nva le vecteur de NumVar
  */
NumVarMatrix::NumVarMatrix( const std::vector<NumVarArray *> & nva )
{
  if( nva.size()!=0 )
  {
    type = nva[0]->GetType();
    for(unsigned i = 0; i < nva.size(); ++i)
        varMatrix.push_back(new NumVarArray(*(nva[i])));
  }
}


/*----------------------------------------------------------------------------*/

/**
  operateur de copie
 */
NumVarMatrix & NumVarMatrix::operator= ( const NumVarMatrix & nva )
{
  type = nva.type;
  for(unsigned i = 0; i < nva.varMatrix.size(); ++i)
    varMatrix.push_back(new NumVarArray(*nva.varMatrix[i]));
  return *this;
}

/**Geter du nombre de colonnes
   @return Le nombre de colonnes de la matrice de variables
*/
unsigned NumVarMatrix::GetNbCols() const
{ 
    if( varMatrix.size() != 0)
        return varMatrix[0]->Size();
    else
        return 0;
}

/**Geter d'une ligne
   @return Le ligne de colonnes de la matrice de variables
*/
NumVarArray NumVarMatrix::GetCol(unsigned j) const
{
  if( varMatrix.size() != 0)
  {
    if( j < varMatrix.size())
    {
        std::vector<NumVar> tmp;
        for(unsigned i=0; i < varMatrix.size(); ++i)
            tmp.push_back( *varMatrix[i]->Get(j));
        return NumVarArray(tmp);
    }
    else
        throw std::out_of_range("NumVarArray::GetRow : index out of range");
  }
  else
    return NumVarArray();
}

/**Geter d'une ligne
   @return La ligne de la matrice de variables
*/
NumVarArray NumVarMatrix::GetRow(unsigned i) const
{ 
    if( varMatrix.size())
        if( i<varMatrix.size() )
            return *(varMatrix[i]);
        else
            throw std::out_of_range("NumVarArray::GetRow : index out of range");
    else
        return NumVarArray();

}


/**Oprateur constant d'indice
   @param i l'indice de la ligne du tableau pour lequel on souhaite rcuprer la variable.
   @param j l'indice de la ligne du tableau pour lequel on souhaite rcuprer la variable.
   @return Une rfrence sur la variable.
*/
NumVar  NumVarMatrix::operator() (unsigned i,unsigned j) const
{
    if( i < varMatrix.size() )
    {
    	if( j < varMatrix[i]->Size() )
            return *(varMatrix[i]->Get(j));
        else
            throw std::out_of_range("NumVarray::operator[] : second index out of range");
   }
    else
        throw std::out_of_range("NumVarray::operator[] : first index out of range");
}

/**Geteur de la variable de ligne i et de colonne j
   @param i l'indice de la ligne du tableau pour lequel on souhaite rcuprer la variable.
   @param j l'indice de la ligne du tableau pour lequel on souhaite rcuprer la variable.
   @return Un pointeur sur la variable.
*/

#ifndef COMPILE_FOR_PYTHON    
NumVar * NumVarMatrix::Get(unsigned i, unsigned j)
{
    if( i < varMatrix.size() )
    {
    	if( j < varMatrix[i]->Size() )
            return varMatrix[i]->Get(j);
        else
            throw std::out_of_range("NumVarray::operator[] : second index out of range");
   }
    else
        throw std::out_of_range("NumVarray::operator[] : first index out of range");
}
#else
NumVar&   NumVarMatrix::Get(unsigned i, unsigned j)
{
    if( i < varMatrix.size() )
    {
    	if( j < varMatrix[i]->Size() )
            return *(varMatrix[i]->Get(j));
        else
            throw std::out_of_range("NumVarray::operator[] : second index out of range");
   }
    else
        throw std::out_of_range("NumVarray::operator[] : first index out of range");
}
#endif

/*----------------------------------------------------------------------------*/

/**
  Destructeur qui dtruit chaque pointeur sur les variables du modles
 */
NumVarMatrix::~NumVarMatrix()
{
  for(unsigned i = 0; i < varMatrix.size(); ++i)
    delete varMatrix[i];
}


#ifdef COMPILE_FOR_PYTHON
       #include "py_NumVarMatrix.cxx"
#endif


}
