/*
	Copyright (c) 2005, Quentin Lequy and Romain Gaucher
	All rights reserved.

	Redistribution and use in source and binary forms, with or 
	without modification, are permitted provided that the following 
	conditions are met:

		* Redistributions of source code must retain the above copyright 
		  notice, this list of conditions and the following disclaimer.

		* Redistributions in binary form must reproduce the above copyright 
		  notice, this list of conditions and the following disclaimer in the 
		  documentation and/or other materials provided with the distribution.

		* Neither the name of Quentin Lequy or Romain Gaucher nor the names 
		  of its contributors may be used to endorse or promote products 
		  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
	AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


#include "Png.h"
#include <png.h>
#include <iostream>

namespace Modelib {

using namespace std;

PNGImage::~PNGImage() 
{
  cerr << "PNG - destructor" << endl;
  if (fp!=NULL)
     fclose(fp);
  if (image)
  {
    for (unsigned i=0;i<height;i++)
          delete [] image[i];
    delete image;
  }
  delete [] row_pointers;
}


bool PNGImage::openFileRead() {
  if (fp!=NULL) {
     cerr << "file already opened\n" << endl;
    return false;	// can't open an already opened file
  }

  fp = fopen(filename.c_str(), "rb");
  if (!fp) {
    cerr << "couldn't open file " << filename << "for reading" << endl;;
    return false;			// couldn't open file
  }
	
  cerr << "opened file " <<  filename << endl;
  return true;
}

bool PNGImage::openFileWrite() {
  if (fp!=NULL) {
    cerr << "file already opened" << endl;
    return false;	// can't open an already opened file
  }

  fp = fopen(filename.c_str(), "wb");
  if (!fp) {
    return false;			// couldn't open file
  }
  return true;
}

bool PNGImage::isPNG(int bytesToCheck = 8) {
  if (fp==NULL) {
    if (openFileRead() == false) return false;
  }

  if (bytesToCheck > 8) {
    bytesToCheck=8;
    cerr << "bytesToCheck set to 8.  It cannot be more than 8." << endl;
  }
  else if (bytesToCheck < 1) {
    bytesToCheck=1;
    cerr << "bytesToCheck set to 1.  It cannot be less than 1." << endl;
  }

  // todo: check that fp is at start of file.
	
  char header[8];	// 8 is than maximum size that can be checked.
	
  int retValue = fread(header, 1, bytesToCheck, fp);
  if (retValue != bytesToCheck) {
    cerr << "couldn't read " << bytesToCheck << "bytes from file" << endl;
  }

  // use library to check if it's a valid PNG file
  bool valid;
  valid = ( png_sig_cmp((png_byte*)header, 0, bytesToCheck) == 0);
	
  return valid;
}


bool PNGImage::initReadStructs() {
  //  printf("initReadStructs: starting..." << endl;

  // initialise png_ptr
  //  printf("initReadStructs: initialising png_ptr..." << endl;
  png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING,    // from png.h
				   NULL, // ptr to struct for user error funct.
				   NULL, // ptr to user error function
				   NULL);// ptr to usr warning function

  if (png_ptr == NULL) {
    cerr << "initReadStructs: png_ptr was not able to be initialised" << endl;
    return false;
  }


  // initialise info_ptr
  //  printf("initReadStructs: initialising info_ptr..." << endl;
  info_ptr = png_create_info_struct(png_ptr);
  if (info_ptr == NULL) {
    cerr << "initReadStructs: info_ptr was not able to be initialised" << endl;
    png_destroy_read_struct(&png_ptr,
			    (png_infopp)NULL,   // ?
			    (png_infopp)NULL);  // ?
    return false;
  }

  // note: there is no end_info variable because I don't know what
  // it's for yet.  But if we need one then it's memory allocation
  // should happen here (once a variable has been declared in the
  // class).

  //  printf("initReadStructs: finished" << endl;
  return true;
}


bool PNGImage::initWriteStructs() {
  png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING,
				    NULL, // ptr to user error struct
				    NULL, // ptr to user error function
				    NULL);// ptr to user warning function

  if (!png_ptr) {
    cerr << "initWriteStructs: failed to init png_ptr" << endl;
    return false;
  }

  info_ptr = png_create_info_struct(png_ptr);
  if (!info_ptr) {
    cerr << "initWriteStructs: failed to init info_ptr" << endl;
    png_destroy_write_struct(&png_ptr,
			     (png_infopp)NULL);
    return false;
  }

  // prepare for errors.  Can't call setjmp() before png_ptr has been
  // allocated because the data inside png_ptr is access by png_jmpbuf!
  if (setjmp(png_jmpbuf(png_ptr))) {  // png_jmpbuf is a macro in pngconf.h
    cerr << "initReadStructs: setjmp returned non-zero (i.e. an error occured.)" << endl;
    png_destroy_read_struct(&png_ptr, &info_ptr, NULL /*end_info*/);
    return false;
  }
  png_init_io(png_ptr, fp);
  cerr << "initWriteStructs: complete" << endl;
  return true;
}


bool PNGImage::writeHeader() {
  cerr << "writeHeader: starting..." << endl;
  // prepare for errors.
  if (setjmp(png_jmpbuf(png_ptr))) {  // png_jmpbuf is a macro in pngconf.h
    cerr << "writeHeader: setjmp returned non-zero (i.e. an error occured.)" << endl;
    png_destroy_read_struct(&png_ptr, &info_ptr, NULL );
    return false;
  }

  cerr << "writeHeader: setting header" << endl;
  png_set_IHDR(png_ptr, info_ptr, width, height,
	       8, PNG_COLOR_TYPE_RGB, PNG_INTERLACE_NONE,
	       PNG_COMPRESSION_TYPE_BASE, PNG_FILTER_TYPE_BASE);

  cerr << "writeHeader: writing header" << endl;
  png_write_info(png_ptr, info_ptr);
  cerr << "writeHeader: finished." << endl;

  return true;
}


png_byte** PNGImage::writeImage_Start() {
  cerr << "writeImage: starting..." << endl;
  cerr << "image is " << height << " by " << width << endl;
  // prepare for errors.
  if (setjmp(png_jmpbuf(png_ptr))) {  // png_jmpbuf is a macro in pngconf.h
    cerr << "writeImage: setjmp returned non-zero (i.e. an error occured.)" << endl;
    png_destroy_read_struct(&png_ptr, &info_ptr, NULL /*end_info*/);
    return false;
  }

  for (unsigned k = 0; k < height; k++) {
    row_pointers[k] = image[k];
  }

  return image;
}


bool PNGImage::writeImage_End() {
  png_write_image(png_ptr, row_pointers);
  cerr << "writeImage: finished." << endl;
  return true;
}

bool PNGImage::writeEnd() {
  cerr << "writeEnd: starting..." << endl;

  // prepare for errors.
  if (setjmp(png_jmpbuf(png_ptr))) {  // png_jmpbuf is a macro in pngconf.h
    cerr << "writeEnd: setjmp returned non-zero (i.e. an error occured.)" << endl;
    png_destroy_read_struct(&png_ptr, &info_ptr, NULL /*end_info*/);
    return false;
  }

  png_write_end(png_ptr, NULL);  // NULL because we don't need to
				 // write any comments, etc.

  cerr << "writeEnd: finished" << endl;
  return true;
}

}

