/*
	Copyright (c) 2005, Quentin Lequy and Romain Gaucher
	All rights reserved.

	Redistribution and use in source and binary forms, with or 
	without modification, are permitted provided that the following 
	conditions are met:

		* Redistributions of source code must retain the above copyright 
		  notice, this list of conditions and the following disclaimer.

		* Redistributions in binary form must reproduce the above copyright 
		  notice, this list of conditions and the following disclaimer in the 
		  documentation and/or other materials provided with the distribution.

		* Neither the name of Quentin Lequy or Romain Gaucher nor the names 
		  of its contributors may be used to endorse or promote products 
		  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
	AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "Model.h"
#include "Utils.h"
#include "System.h"

// Les diffrents solveurs implments
#include "GlpkSolver.h"
#include "CplexSolver.h"

#include <iostream>
#include <stdexcept>

namespace Modelib {


using namespace std;

/**
	Crer une instance d'un solveur
*/
void SolverAPI::AllocSolver(const SolverName api)
{
	switch(api)
	{
		case GLPK|GLPSOL: 
			solver = new GlpkSolver(); 
			break;
		
		case CPLEX:
			solver = new CplexSolver();
			break;

		default:
			solver = 0;
			throw "(SolverAPI::AllocSolver) - Allocation impossible, solveur inconnu";
			break;
	}
}


/**
	Constructeur de notre interface
	
	@param api Le nom du solveur a utiliser (cf enumrateur SolverName)
*/
SolverAPI::SolverAPI(const SolverName api)
  : apiSolver(api), solver(0)
{
	fileLP = "temp.lp";
	try {
		AllocSolver(api);
	}
	catch (string& e) {
		cout << e << endl;
	}
}

/**
	Destructeur
*/
SolverAPI::~SolverAPI()
{
	if (solver) delete solver;
}

/**
	Initialisation du chemin du solver et du nom du fichier temporaire
	
	@param tempFile Le nom du fichier temporaire
	@param _path    Le chemin du solveur, la commande sera "execute _path + exeSolver"
*/
void SolverAPI::Init(const std::string& tempFile, const std::string& _path)
{
	fileLP = tempFile;
	path   = _path;
}

/**
	Initialisation du chemin du solver et du nom du fichier temporaire et du solveur
	
	@param api      Le solveur  utiliser
	@param tempFile Le nom du fichier temporaire
	@param _path    Le chemin du solveur, la commande sera "execute _path + exeSolver"
*/

void SolverAPI::Init(const SolverName api, const std::string& tempFile, const std::string& _path)
{
	SetAPI(api);
	fileLP = tempFile;
	path   = _path;
}

/**
	Attribuer une nouvelle API
*/
void SolverAPI::SetAPI(const SolverName api)
{
	solution.Clear();
	if (apiSolver != api)
	{
		apiSolver = api;
		delete solver;
		try {
			AllocSolver(api);
		}
		catch (string& e)
		{
			cout << e << endl;
		}
	}
}

}
